//************************************************************************************************
// Fast AVR FFT - some demo functions
//************************************************************************************************
// Fast FFT routines in 16-bit fixed point resolution. Cooley-Tukey DIT radix 2 algorithm.  
// All functions uses common global complex work array:
//   int16_t FFTwarr[FFTN*2]// 
// where FFTN is FFT size, memory organisation in array:
//   re{0},im{0},re{1},im{1},...,re{FFTN-1},im{FFTN-1}
// Routines should work for FFTN={32,64,128,256,512}, lower resolutions are also possible, 
// but there are no lookups generated for them.
//
// Usage:
// ------
// 1. fill im{} items of FFTwarr[] with signal to process
// 2. call function FFT_input()
//     - this function does bit reversion and applies selected window      
//     - then optionally duplicates re{} to im{} to increase amplitude by 2^0.5 (better SNR) 
// 3. call FFT_dit_r2() to do main FFT DIT radix 2 transformation
//     - complex results are stored in FFTwarr and are already in correct order
// 4. call FFT_out_lin() or FFT_out_log() to create linear or log scale amplitude spectrum from 
//    first half of complex spectrum, amplitudes "r" are stored into FFTwarr[] in this order:
//      FFTwarr[0]=r[0],FFTwarr[1]=r[1],FFTwarr[2]=r[2],... 
//
// Options (FFT.h):
// ----------------
// FFTN     - FFT size	     
// FFTW     - window to apply
// FFTdupim - duplicate input signal to both re{} and im{} part of FFTwarr[]
//          - this increases input signal amplitude by 2^0.5 so it'll have better SNR
// FFTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
// FFTopt   - increases algorithm performance (bigger code size)
// FFTfsqrt - uses fast lookup sqrt() insted of precise one
//
// Files:
// ------
// Whole FFT library consist of several files:
// 1. FFT_routines.s - main FFT algorithm routines
// 2. FFT_macros.inc - some higher level FFT related macros
// 3. FFT_tables.inc - FFT lookups
// 4. FFT_math.s     - FFT related arithmetics
// 5. FFT.c          - global FFTwarr[] array and some demo functions
// 6. FFT.h          - common FFT library header file, algorithm parameters
// 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
//
//
//
// Measured execution times for FFTfsqrt=0, FFTprec=0, FFTdupim=1, XTAL=16MHz:
//
// FFTopt=0, linear output:
// FFTN   input  transformation  output  total   performance
// [-]    [ms]   [ms]            [ms]    [ms]    [kp/s]
// 32     0.118   0.707          0.601    1.426  22.440
// 64     0.235   1.670          1.203    3.108  20.592
// 128    0.485   3.855          2.410    6.750  18.963
// 256    0.969   8.745          4.821   14.535  17.613
// 512    1.937  19.567          9.643   31.147  16.438
//
// FFTopt=1, linear output:
// FFTN   input  transformation  output  total   performance
// [-]    [ms]   [ms]            [ms]    [ms]    [kp/s]
// 32     0.118   0.500          0.601    1.218  26.273
// 64     0.235   1.251          1.204    2.690  23.796
// 128    0.485   3.012          2.409    5.906  21.673
// 256    0.969   7.057          4.820   12.845  19.930
// 512    1.937  16.190          9.643   27.769  18.437
//
// FFTopt=0, log scale output:
// FFTN   input  transformation  output   total   performance
// [-]    [ms]   [ms]            [ms]     [ms]    [kp/s]
// 32     0.118   0.707           0.655    1.480  21.622
// 64     0.235   1.670           1.305    3.210  19.935
// 128    0.485   3.855           2.613    6.953  18.408
// 256    0.969   8.745           5.227   14.941  17.143
// 512    1.937  19.567          10.458   31.962  16.019
//
// FFTopt=1, log scale output:
// FFTN   input  transformation  output   total   performance
// [-]    [ms]   [ms]            [ms]     [ms]    [kp/s]
// 32     0.118   0.500           0.655    1.272  25.157
// 64     0.235   1.250           1.306    2.791  22.931
// 128    0.485   3.012           2.609    6.106  20.963
// 256    0.969   7.056           5.216   13.241  19.334
// 512    1.937  16.190          10.431   28.558  17.928
//
// Performance for FFTprec=1 is 12% lower or so. 
//
// (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
// http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFFT/AVRFFT.html
//************************************************************************************************
#include "avr/io.h"

#include "FFT.h"
#include "serial.h"


// global FFT work array
uint16_t FFTwarr[FFTN*2];




// ------ recieve data to process ------
void FFT_rx_data(void)
{
	// read data block into work array
	uint8_t *wr=(uint8_t*)&FFTwarr[1];
	do{
		// store to im{}
		*wr++=rx_byte();
		*wr++=rx_byte();

		// skip re{} part
		wr+=2;
	}while(wr<(uint8_t*)&FFTwarr[FFTN*2]);
}


// ------ send processed data ------
void FFT_tx_data(uint8_t mode)
{
	if(mode)
	{
		// --- frequency domain ---

		// send first half of complex spectrum data
		uint16_t *rd=&FFTwarr[0];
		do{
			tx_word(*rd++);
		}while(rd<&FFTwarr[FFTN]);
	}
	else
	{
		// --- time domain ---
		
		// send real part of time domain data
		uint16_t *rd=&FFTwarr[0];
		do{
			tx_word(*rd);
			rd+=2;
		}while(rd<&FFTwarr[FFTN*2]);
	}

}
