;************************************************************************************************
; Fast AVR FFT - FFT related macros
;************************************************************************************************
; Fast FFT routines in 16-bit fixed point resolution. Cooley-Tukey DIT radix 2 algorithm.  
; All functions uses common global complex work array:
;   int16_t FFTwarr[FFTN*2]; 
; where FFTN is FFT size, memory organisation in array:
;   re{0},im{0},re{1},im{1},...,re{FFTN-1},im{FFTN-1}
; Routines should work for FFTN={32,64,128,256,512}, lower resolutions are also possible, 
; but there are no lookups generated for them.
;
; Usage:
; ------
; 1. fill im{} items of FFTwarr[] with signal to process
; 2. call function FFT_input()
;     - this function does bit reversion and applies selected window      
;     - then optionally duplicates re{} to im{} to increase amplitude by 2^0.5 (better SNR) 
; 3. call FFT_dit_r2() to do main FFT DIT radix 2 transformation
;     - complex results are stored in FFTwarr and are already in correct order
; 4. call FFT_out_lin() or FFT_out_log() to create linear or log scale amplitude spectrum from 
;    first half of complex spectrum, amplitudes "r" are stored into FFTwarr[] in this order:
;      FFTwarr[0]=r[0],FFTwarr[1]=r[1],FFTwarr[2]=r[2],... 
;
; Options (FFT.h):
; ----------------
; FFTN     - FFT size	     
; FFTW     - window to apply
; FFTdupim - duplicate input signal to both re{} and im{} part of FFTwarr[]
;          - this increases input signal amplitude by 2^0.5 so it'll have better SNR
; FFTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
; FFTopt   - increases algorithm performance (bigger code size)
; FFTfsqrt - uses fast lookup sqrt() insted of precise one
;
; Files:
; ------
; Whole FFT library consist of several files:
; 1. FFT_routines.s - main FFT algorithm routines
; 2. FFT_macros.inc - some higher level FFT related macros
; 3. FFT_tables.inc - FFT lookups
; 4. FFT_math.s     - FFT related arithmetics
; 5. FFT.c          - global FFTwarr[] array and some demo functions
; 6. FFT.h          - common FFT library header file, algorithm parameters
; 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
;
;
;
; Measured execution times for FFTfsqrt=0, FFTprec=0, FFTdupim=1, XTAL=16MHz:
;
; FFTopt=0, linear output:
; FFTN   input  transformation  output  total   performance
; [-]    [ms]   [ms]            [ms]    [ms]    [kp/s]
; 32     0.118   0.707          0.601    1.426  22.440
; 64     0.235   1.670          1.203    3.108  20.592
; 128    0.485   3.855          2.410    6.750  18.963
; 256    0.969   8.745          4.821   14.535  17.613
; 512    1.937  19.567          9.643   31.147  16.438
;
; FFTopt=1, linear output:
; FFTN   input  transformation  output  total   performance
; [-]    [ms]   [ms]            [ms]    [ms]    [kp/s]
; 32     0.118   0.500          0.601    1.218  26.273
; 64     0.235   1.251          1.204    2.690  23.796
; 128    0.485   3.012          2.409    5.906  21.673
; 256    0.969   7.057          4.820   12.845  19.930
; 512    1.937  16.190          9.643   27.769  18.437
;
; FFTopt=0, log scale output:
; FFTN   input  transformation  output   total   performance
; [-]    [ms]   [ms]            [ms]     [ms]    [kp/s]
; 32     0.118   0.707           0.655    1.480  21.622
; 64     0.235   1.670           1.305    3.210  19.935
; 128    0.485   3.855           2.613    6.953  18.408
; 256    0.969   8.745           5.227   14.941  17.143
; 512    1.937  19.567          10.458   31.962  16.019
;
; FFTopt=1, log scale output:
; FFTN   input  transformation  output   total   performance
; [-]    [ms]   [ms]            [ms]     [ms]    [kp/s]
; 32     0.118   0.500           0.655    1.272  25.157
; 64     0.235   1.250           1.306    2.791  22.931
; 128    0.485   3.012           2.609    6.106  20.963
; 256    0.969   7.056           5.216   13.241  19.334
; 512    1.937  16.190          10.431   28.558  17.928
;
; Performance for FFTprec=1 is 12% lower or so. 
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFFT/AVRFFT.html
;************************************************************************************************


	;@1 <= (signed(@0)*signed(@0))<<1
	;32 <= 16*16
	;using @2 as zero reg
	.macro	pwrsw ra,rb,pa 
	findreg2 \ra,\rb
	fmuls	(.findreg_ra)+1,(.findreg_ra)+1
	movw	(.findreg_rb)+2,R0
	fmul	(.findreg_ra),(.findreg_ra)
	adc	(.findreg_rb)+2,\pa
	movw	(.findreg_rb),R0
	fmulsu	(.findreg_ra)+1,(.findreg_ra)
	brcc	.+(1+1)*2
	dec	(.findreg_rb)+3
	dec	(.findreg_rb)+3
	add	(.findreg_rb)+1,R0
	adc	(.findreg_rb)+2,R1
	adc	(.findreg_rb)+3,\pa
	add	(.findreg_rb)+1,R0
	adc	(.findreg_rb)+2,R1
	adc	(.findreg_rb)+3,\pa
	.endm
	
	;@2 <= (signed(@0)*signed(@1))<<1
	;32 <= (16*16)<<1
	;using @3 as zero reg
	.macro	fmulsw ra,rb,rc,pa 
	findreg3 \ra,\rb,\rc
	fmuls	(.findreg_ra)+1,(.findreg_rb)+1
	movw	(.findreg_rc)+2,R0
	fmul	(.findreg_ra),(.findreg_rb)
	adc	(.findreg_rc)+2,\pa
	movw	(.findreg_rc),R0
	fmulsu	(.findreg_ra)+1,(.findreg_rb)
	sbc	(.findreg_rc)+3,\pa
	add	(.findreg_rc)+1,R0
	adc	(.findreg_rc)+2,R1
	adc	(.findreg_rc)+3,\pa
	fmulsu	(.findreg_rb)+1,(.findreg_ra)
	sbc	(.findreg_rc)+3,\pa
	add	(.findreg_rc)+1,R0
	adc	(.findreg_rc)+2,R1
	adc	(.findreg_rc)+3,\pa	
	.endm
	
	;@2 <= signed(@0)*signed(@1)
	;32 <= 16*16
	;using @3 as zero reg
	.macro	mulsw ra,rb,rc,pa 
	findreg3 \ra,\rb,\rc
	muls	(.findreg_ra)+1,(.findreg_rb)+1
	movw	(.findreg_rc)+2,R0
	mul	(.findreg_ra),(.findreg_rb)
	movw	(.findreg_rc),R0
	mulsu	(.findreg_ra)+1,(.findreg_rb)
	sbc	(.findreg_rc)+3,\pa
	add	(.findreg_rc)+1,R0
	adc	(.findreg_rc)+2,R1
	adc	(.findreg_rc)+3,\pa
	mulsu	(.findreg_rb)+1,(.findreg_ra)
	sbc	(.findreg_rc)+3,\pa
	add	(.findreg_rc)+1,R0
	adc	(.findreg_rc)+2,R1
	adc	(.findreg_rc)+3,\pa	
	.endm
	
	;@2 <= signed(@0)*unsigned(@1)
	;32 <= 16*16
	;using @3 as zero reg
	.macro	mulsuw ra,rb,rc,pa 
	findreg3 \ra,\rb,\rc
	mulsu	(.findreg_ra)+1,(.findreg_rb)+1
	movw	(.findreg_rc)+2,R0
	mul	(.findreg_ra),(.findreg_rb)
	movw	(.findreg_rc),R0
	mulsu	(.findreg_ra)+1,(.findreg_rb)
	sbc	(.findreg_rc)+3,\pa
	add	(.findreg_rc)+1,R0
	adc	(.findreg_rc)+2,R1
	adc	(.findreg_rc)+3,\pa
	mul	(.findreg_rb)+1,(.findreg_ra)
	add	(.findreg_rc)+1,R0
	adc	(.findreg_rc)+2,R1
	adc	(.findreg_rc)+3,\pa	
	.endm

	;signed(@5:@6) <= signed(@0)*signed(@1) - signed(@2)*unsigned(@3)
	;32 <= 16*16 - 16*16
	;using @7 as zero reg, and changing @6 register PAIR 
	.macro	msubw ra,rb,rc,rd,re,rf,rg,pa 
	findreg7 \ra,\rb,\rc,\rd,\re,\rf,\rg
	muls	(.findreg_ra)+1,(.findreg_rb)+1
	movw	(.findreg_rf),R0
	mulsu	(.findreg_rc)+1,(.findreg_rd)+1	;
	movw	(.findreg_rg),R0
	mul	(.findreg_ra),(.findreg_rb)
	movw	(.findreg_re),R0
	mul	(.findreg_rc),(.findreg_rd)	;
	sub	(.findreg_re),R0
	sbc	(.findreg_re)+1,R1
	sbc	(.findreg_rf),(.findreg_rg)
	sbc	(.findreg_rf)+1,(.findreg_rg)+1
	mulsu	(.findreg_ra)+1,(.findreg_rb)
	sbc	(.findreg_rf)+1,\pa
	add	(.findreg_re)+1,R0
	adc	(.findreg_rf),R1
	adc	(.findreg_rf)+1,\pa
	mulsu	(.findreg_rc)+1,(.findreg_rd)	;
	adc	(.findreg_rf)+1,\pa
	sub	(.findreg_re)+1,R0
	sbc	(.findreg_rf),R1
	sbc	(.findreg_rf)+1,\pa
	mulsu	(.findreg_rb)+1,(.findreg_ra)
	sbc	(.findreg_rf)+1,\pa
	add	(.findreg_re)+1,R0
	adc	(.findreg_rf),R1
	adc	(.findreg_rf)+1,\pa
	mul	(.findreg_rd)+1,(.findreg_rc)	;
	sub	(.findreg_re)+1,R0
	sbc	(.findreg_rf),R1
	sbc	(.findreg_rf)+1,\pa
	.endm
	
	;signed(@5:@6) <= signed(@0)*signed(@1) + signed(@2)*unsigned(@3)
	;32 <= 16*16 + 16*16
	;using @7 as zero reg, and changing @6 register PAIR
	.macro	maddw ra,rb,rc,rd,re,rf,rg,pa 
	findreg7 \ra,\rb,\rc,\rd,\re,\rf,\rg
	muls	(.findreg_ra)+1,(.findreg_rb)+1
	movw	(.findreg_rf),R0
	mulsu	(.findreg_rc)+1,(.findreg_rd)+1	;
	movw	(.findreg_rg),R0
	mul	(.findreg_ra),(.findreg_rb)
	movw	(.findreg_re),R0
	mul	(.findreg_rc),(.findreg_rd)	;
	add	(.findreg_re),R0
	adc	(.findreg_re)+1,R1
	adc	(.findreg_rf),(.findreg_rg)
	adc	(.findreg_rf)+1,(.findreg_rg)+1
	mulsu	(.findreg_ra)+1,(.findreg_rb)
	sbc	(.findreg_rf)+1,\pa
	add	(.findreg_re)+1,R0
	adc	(.findreg_rf),R1
	adc	(.findreg_rf)+1,\pa
	mulsu	(.findreg_rc)+1,(.findreg_rd)	;
	sbc	(.findreg_rf)+1,\pa
	add	(.findreg_re)+1,R0
	adc	(.findreg_rf),R1
	adc	(.findreg_rf)+1,\pa
	mulsu	(.findreg_rb)+1,(.findreg_ra)
	sbc	(.findreg_rf)+1,\pa
	add	(.findreg_re)+1,R0
	adc	(.findreg_rf),R1
	adc	(.findreg_rf)+1,\pa
	mul	(.findreg_rd)+1,(.findreg_rc)	;
	add	(.findreg_re)+1,R0
	adc	(.findreg_rf),R1
	adc	(.findreg_rf)+1,\pa
	.endm
	
	
;------------------------------------------------------------------------------------------------
;	simplified butterfly algorithm, called from "FFTditr2"
;	algorithm is optimized for ang=0 where original algorithm: 
;	A_new <= A_old + Fki*B_old
;	B_new <= A_old - Fki*B_old
;	
;	can be simplified as:
;	A_new <= A_old + B_old
;	B_new <= A_old - B_old
;------------------------------------------------------------------------------------------------
	;allow signed correction
	.equ	FFTditr2a0Mp,1
	
	.macro	FFTditr2a0M 
	mov	R13,R15	;butterfly counter
	;--- for each butterfly ---
FFTditr2a0ML:	ld	R8,X+
	ld	R9,X+
	asr	R9
	ror	R8	;Rre=re{*X}>>1
	.if	FFTprec && FFTditr2a0Mp
	tst	R9
	brpl	.+2*2
	adc	R8,R2
	adc	R9,R2
	.endif
	ld	R6,X+
	ld	R7,X+
	asr	R7
	ror	R6	;Rim=im{*X}>>1
	.if	FFTprec && FFTditr2a0Mp
	tst	R7
	brpl	.+2*2
	adc	R6,R2
	adc	R7,R2
	.endif
	ldd	R16,Y+0
	ldd	R17,Y+1
	asr	R17
	ror	R16	;Are=re{*Y}>>1
	.if	FFTprec && FFTditr2a0Mp
	tst	R17
	brpl	.+2*2
	adc	R16,R2
	adc	R17,R2
	.endif
	ldd	R18,Y+2
	ldd	R19,Y+3
	asr	R19
	ror	R18	;Aim=im{*Y}>>1
	.if	FFTprec && FFTditr2a0Mp
	tst	R19
	brpl	.+2*2
	adc	R18,R2
	adc	R19,R2
	.endif
	movw	R4,R16
	movw	R10,R18	;backup Are,Aim
	;A output
	add	R16,R8
	adc	R17,R9
	std	Y+0,R16
	std	Y+1,R17	;re{*Y}=Are+Rre
	add	R18,R6
	adc	R19,R7
	std	Y+2,R18
	std	Y+3,R19	;im{*y}=Aim+Rim
	;B output
	sub	R10,R6
	sbc	R11,R7
	st	-X,R11
	st	-X,R10	;im{*X}=Aim-Rim
	sub	R4,R8
	sbc	R5,R9
	st	-X,R5
	st	-X,R4	;re{*X}=Are-Rre
	;move input pointers to next butterfly inputs
	add	R28,R24
	adc	R29,R25	;Y+=2*n
	add	R26,R24
	adc	R27,R25	;X+=2*n
	dec	R13
	brne	FFTditr2a0ML	;next butterfly	
	.endm
	
	
;------------------------------------------------------------------------------------------------
;	simplified butterfly algorithm, called from "FFTditr2"
;	algorithm is optimized for ang=pi/2 where original algorithm: 
;	A_new <= A_old + Fki*B_old
;	B_new <= A_old - Fki*B_old
;	
;	can be simplified as:
;	re{A_new} <= re{A_old} + im{B_old}
;	im{A_new} <= im{A_old} - re{B_old}
;	re{B_new} <= re{A_old} - im{B_old}
;	im{B_new} <= im{A_old} + re{B_old}
;------------------------------------------------------------------------------------------------
	;allow signed corection
	.equ	FFTditr2api2Mp,1
	
	.macro	FFTditr2api2M 
	mov	R13,R15	;butterfly counter
	;--- for each butterfly ---
FFTditr2api2ML:	ld	R8,X+
	ld	R9,X+
	asr	R9
	ror	R8	;Rre=re{*X}>>1
	.if	FFTprec && FFTditr2api2Mp
	tst	R9
	brpl	.+2*2
	adc	R8,R2
	adc	R9,R2
	.endif
	ld	R6,X+
	ld	R7,X+
	asr	R7	
	ror	R6	;Rim=im{*X}>>1
	.if	FFTprec && FFTditr2api2Mp
	tst	R7
	brpl	.+2*2
	adc	R6,R2
	adc	R7,R2
	.endif
	ldd	R16,Y+0
	ldd	R17,Y+1
	asr	R17
	ror	R16	;Are=re{*Y}>>1
	.if	FFTprec && FFTditr2api2Mp
	tst	R17
	brpl	.+2*2
	adc	R16,R2
	adc	R17,R2
	.endif
	ldd	R18,Y+2
	ldd	R19,Y+3
	asr	R19	
	ror	R18	;Aim=im{*Y}>>1
	.if	FFTprec && FFTditr2api2Mp
	tst	R19
	brpl	.+2*2
	adc	R18,R2
	adc	R19,R2
	.endif
	movw	R4,R16
	movw	R10,R18	;backup Are,Aim
	;A output
	add	R16,R6
	adc	R17,R7
	std	Y+0,R16	
	std	Y+1,R17	;re{*Y}=Are-Rim
	sub	R18,R8
	sbc	R19,R9
	std	Y+2,R18
	std	Y+3,R19	;im{*y}=Aim+Rre
	;B output
	add	R10,R8
	adc	R11,R9
	st	-X,R11
	st	-X,R10	;im{*X}=Aim-Rre
	sub	R4,R6
	sbc	R5,R7
	st	-X,R5
	st	-X,R4	;re{*X}=Are+Rim
	;move input pointers to next butterfly inputs
	add	R28,R24
	adc	R29,R25	;Y+=2*n
	add	R26,R24
	adc	R27,R25	;X+=2*n
	dec	R13
	brne	FFTditr2api2ML	;next butterfly	
	.endm
