//************************************************************************************************
// Fast Harltey Transform (FHT) for AVR - some demo functions
//************************************************************************************************
// Fast FHT routines in 16-bit fixed point resolution. Implementation is based on Ullmann's paper
// from 1987 (good description and some examples of FFT and FHT):
//   http://sepwww.stanford.edu/theses/sep38/38_29.pdf  
//
// As expected the main transform needs only some 55% CPU time compared to FFT in the same
// resolution with the same optimalization techniques. 
// Overall performance is only 140% compared to FFT because amplitude spectrum output uses 
// accurate sqrt32() from ChanN. With log scale output the results are much better since it's 
// done using lookups (160% of FFT performance).
// But the main reason I made this routines is to save RAM because FHT uses real only work array.   
//
// All functions uses common global work array:
//   int16_t FHTwarr[FHTN];
// where FHTN is FHT size. Routines should work for FHTN={32,64,128,256,512}, lower resolutions 
// are also possible, but there are no lookups generated for them. 
//
//
// 1. Usage
// --------
// 1. call function FHT_input(int16_t *src)
//     - this function fills work array with bitreversed and windowed src data 
// 3. call FHT_do() to do the main FHT transform
//     - image is stored into the global work array FHTwarr[] already in order
// 4. call FHT_out_lin() or FHT_out_log() to create half of linear or log scale amplitude 
//    spectrum from FHT image. Results are stored again into FHTwarr[] array. 
//
//
// 2. Options (FHT.h)
// ------------------
// FHTN     - FHT size	     
// FHTW     - window to apply
// FHTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
// FHTfsqrt - uses fast lookup sqrt() instead of accurate one
//
//
// 3. Files
// --------
// Whole FHT library consist of several files:
// 1. FHT_routines.s - main FHT algorithm routines
// 2. FHT_macros.inc - some higher level FHT related macros
// 3. FHT_tables.inc - FHT lookups
// 4. FHT_math.s     - FHT related arithmetics
// 5. FHT.c          - global FHTwarr[] array and some demo functions
// 6. FHT.h          - common FHT library header file, algorithm parameters
// 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
//
//
// 4. Performance
// --------------
// Measured execution times for FHTfsqrt=0, XTAL=16MHz:
//
// FHTprec=0, linear amplitude output:
// FFTN  input   transform  output  total    performance
// [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
// 32    0.096   0.293      0.606    0.995   32.16
// 64    0.188   0.714      1.201    2.103   30.43
// 128   0.371   1.692      2.406    4.469   28.64
// 256   0.771   3.919      4.810    9.500   26.95
// 512   1.539   8.972      9.626   20.137   25.43
//
// FHTprec=1, linear amplitude output:
// FFTN  input   transform  output  total    performance
// [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
// 32    0.096   0.327      0.606    1.029   32.16
// 64    0.188   0.798      1.201    2.187   30.43
// 128   0.371   1.876      2.406    4.653   28.64
// 256   0.771   4.325      4.810    9.906   26.95
// 512   1.539   9.876      9.626   21.041   25.43
//
// Output times for log scale spectrum less than 25% compared to linear scale:
// FFTN  output
// [-]   [ms]
// 32    0.139
// 64    0.273
// 128   0.541 
// 256   1.076
// 512   2.148
//
//
// 5. Licence
// ----------
// It's free for any projects you want but without any warranty since it's free and open.
//
//
// (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
// http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
//************************************************************************************************
#include "avr/io.h"
#include <avr/pgmspace.h>

#include "FHT.h"
#include "serial.h"


// global FHT work array
int16_t FHTwarr[FHTN];


//------------------------------------------------------------------------------------------------
// Recieve data via usart, apply window, store bitreversed into work array FHTwarr[].
// Usart data source can be repladed with anything else, it't just demo function to show 
// how to fill work array ...
// 
// If source data are alrady available in some array then rather use assembler function:
//   FHT_input(int16_t *src);
//------------------------------------------------------------------------------------------------
// --- 16=16x16 fractional signed mutliplication ---
#define fmul_16(_a,_b)                        \
({                                        \
	int16_t _c;                           \
	uint8_t _tmp;                         \
	uint8_t _zero;                        \
	asm(		                          \
		"clr	%[z]"			"\n\t"    \
		"fmuls	%B[a],%B[b]"	"\n\t"    \
		"movw	%A[c],R0"		"\n\t"    \
		"fmul	%A[a],%A[b]"	"\n\t"    \
		"mov	%[tmp],R1"		"\n\t"    \
		"adc	%A[c],%[z]"		"\n\t"    \
		"fmulsu	%B[a],%A[b]"	"\n\t"    \
		"sbc	%B[c],%[z]"		"\n\t"    \
		"add	%[tmp],R0"		"\n\t"    \
		"adc	%A[c],R1"		"\n\t"    \
		"adc	%B[c],%[z]"		"\n\t"    \
		"fmulsu	%B[b],%A[a]"	"\n\t"    \
		"sbc	%B[c],%[z]"		"\n\t"    \
		"add	%[tmp],R0"		"\n\t"    \
		"adc	%A[c],R1"		"\n\t"    \
		"adc	%B[c],%[z]"		"\n\t"    \
		"clr    __zero_reg__"             \
		: [c] "=r" (_c), [tmp] "=&r" (_tmp), [z] "=&r" (_zero)  \
		: [a] "a" ((int16_t)(_a)), [b] "a" ((int16_t)(_b)));    \
	_c;  \
})

// --- main function ---
void FHT_rx_input(void)
{
	// bit reversion lookup
	#if FHTN<256
	const uint8_t *brv=TFHTbrv;
	#else
	const uint16_t *brv=TFHTbrv;
	#endif

	// window lookup
	#if FHTW!=wRECT
	const int16_t *wdw=TFHTwdw;
	#endif

	uint16_t N=FHTN;
	do{
		// get source data word
		int16_t x=(int16_t)rx_word();
		
		#if FHTW!=wRECT

		// get window word
		int16_t w=(int16_t)pgm_read_word(wdw++);

		// apply window to data
		x=fmul_16(x,w);

		#endif
		
		// and store to bit reversed address using lookup
		#if FHTN<256
		*(int16_t*)((int8_t*)FHTwarr+pgm_read_byte(brv++))=x;
		#else
		*(int16_t*)((int8_t*)FHTwarr+pgm_read_word(brv++))=x;
		#endif

	// next round
	}while(--N);
}




//------------------------------------------------------------------------------------------------
// Recieve data to process to some array
//------------------------------------------------------------------------------------------------
void FHT_rx_data(uint8_t *data)
{
	uint16_t N=FHTN*2;
	do{
		*data++=rx_byte();
	}while(--N);
}	




//------------------------------------------------------------------------------------------------
// Send processed data out
//------------------------------------------------------------------------------------------------
void FHT_tx_data(void)
{
	// send work array out
	int16_t *rd=&FHTwarr[0];
	do{
		tx_word(*rd++);
	}while(rd<&FHTwarr[FHTN]);
}
