//************************************************************************************************
// Fast Harltey Transform (FHT) for AVR - common header file
//************************************************************************************************
// Fast FHT routines in 16-bit fixed point resolution. Implementation is based on Ullmann's paper
// from 1987 (good description and some examples of FFT and FHT):
//   http://sepwww.stanford.edu/theses/sep38/38_29.pdf  
//
// As expected the main transform needs only some 55% CPU time compared to FFT in the same
// resolution with the same optimalization techniques. 
// Overall performance is only 140% compared to FFT because amplitude spectrum output uses 
// accurate sqrt32() from ChanN. With log scale output the results are much better since it's 
// done using lookups (160% of FFT performance).
// But the main reason I made this routines is to save RAM because FHT uses real only work array.   
//
// All functions uses common global work array:
//   int16_t FHTwarr[FHTN];
// where FHTN is FHT size. Routines should work for FHTN={32,64,128,256,512}, lower resolutions 
// are also possible, but there are no lookups generated for them. 
//
//
// 1. Usage
// --------
// 1. call function FHT_input(int16_t *src)
//     - this function fills work array with bitreversed and windowed src data 
// 3. call FHT_do() to do the main FHT transform
//     - image is stored into the global work array FHTwarr[] already in order
// 4. call FHT_out_lin() or FHT_out_log() to create half of linear or log scale amplitude 
//    spectrum from FHT image. Results are stored again into FHTwarr[] array. 
//
//
// 2. Options (FHT.h)
// ------------------
// FHTN     - FHT size	     
// FHTW     - window to apply
// FHTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
// FHTfsqrt - uses fast lookup sqrt() instead of accurate one
//
//
// 3. Files
// --------
// Whole FHT library consist of several files:
// 1. FHT_routines.s - main FHT algorithm routines
// 2. FHT_macros.inc - some higher level FHT related macros
// 3. FHT_tables.inc - FHT lookups
// 4. FHT_math.s     - FHT related arithmetics
// 5. FHT.c          - global FHTwarr[] array and some demo functions
// 6. FHT.h          - common FHT library header file, algorithm parameters
// 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
//
//
// 4. Performance
// --------------
// Measured execution times for FHTfsqrt=0, XTAL=16MHz:
//
// FHTprec=0, linear amplitude output:
// FFTN  input   transform  output  total    performance
// [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
// 32    0.096   0.293      0.606    0.995   32.16
// 64    0.188   0.714      1.201    2.103   30.43
// 128   0.371   1.692      2.406    4.469   28.64
// 256   0.771   3.919      4.810    9.500   26.95
// 512   1.539   8.972      9.626   20.137   25.43
//
// FHTprec=1, linear amplitude output:
// FFTN  input   transform  output  total    performance
// [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
// 32    0.096   0.327      0.606    1.029   32.16
// 64    0.188   0.798      1.201    2.187   30.43
// 128   0.371   1.876      2.406    4.653   28.64
// 256   0.771   4.325      4.810    9.906   26.95
// 512   1.539   9.876      9.626   21.041   25.43
//
// Output times for log scale spectrum less than 25% compared to linear scale:
// FFTN  output
// [-]   [ms]
// 32    0.139
// 64    0.273
// 128   0.541 
// 256   1.076
// 512   2.148
//
//
// 5. Licence
// ----------
// It's free for any projects you want but without any warranty since it's free and open.
//
//
// (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
// http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
//************************************************************************************************
#ifndef FHT_H
#define FHT_H

// window types
#define wHANN 0
#define wHAMM 1
#define wBLAC 2
#define wFLAT 3
#define wRECT 4


// ====== FHT SETUP ======
// FHT size {32, 64, 128, 256, 512}
#define FHTN 256

// window type {wHANN-hann, wHAMM-hamming, wBLAC-blacman, wFLAC-flattop, wRECT-rectangular (none)}
#define FHTW wHANN

// increase FHT precision (reduces noise)
#define FHTprec 0

// use faster but lower precision sqrt()
#define FHTfsqrt 0




#ifndef __ASSEMBLER__

// common work array
extern int16_t FHTwarr[FHTN];

// bit reversion table
#if FHTN<256
const uint8_t TFHTbrv[FHTN];
#else
const uint16_t TFHTbrv[FHTN];
#endif
// window table
const int16_t TFHTwdw[FHTN];

// main FHT library functions
extern void FHT_rx_input(void);
extern void FHT_input(uint8_t *data);
extern void FHT_do(void);
extern void FHT_out_lin(void);
extern void FHT_out_log(void);

// GCC wrappers for FHT arithemtics functions
extern uint16_t FHT_sqrt32c(uint32_t x);
extern uint16_t FHT_sqrt32f(uint32_t x);
extern int16_t FHT_s32log10f(uint32_t x);
extern int16_t FHT_log10f(uint16_t x);

// some demo functions
extern void FHT_rx_data(uint8_t *data);
extern void FHT_tx_data(void);

#endif
#endif
