;************************************************************************************************
; Fast Harltey Transform (FHT) for AVR - FHT related math routines
;************************************************************************************************
; Fast FHT routines in 16-bit fixed point resolution. Implementation is based on Ullmann's paper
; from 1987 (good description and some examples of FFT and FHT):
;   http://sepwww.stanford.edu/theses/sep38/38_29.pdf  
;
; As expected the main transform needs only some 55% CPU time compared to FFT in the same
; resolution with the same optimalization techniques. 
; Overall performance is only 140% compared to FFT because amplitude spectrum output uses 
; accurate sqrt32() from ChanN. With log scale output the results are much better since it's 
; done using lookups (160% of FFT performance).
; But the main reason I made this routines is to save RAM because FHT uses real only work array.   
;
; All functions uses common global work array:
;   int16_t FHTwarr[FHTN]; 
; where FHTN is FHT size. Routines should work for FHTN={32,64,128,256,512}, lower resolutions 
; are also possible, but there are no lookups generated for them. 
;
;
; 1. Usage
; --------
; 1. call function FHT_input(int16_t *src)
;     - this function fills work array with bitreversed and windowed src data 
; 3. call FHT_do() to do the main FHT transform
;     - image is stored into the global work array FHTwarr[] already in order
; 4. call FHT_out_lin() or FHT_out_log() to create half of linear or log scale amplitude 
;    spectrum from FHT image. Results are stored again into FHTwarr[] array. 
;
;
; 2. Options (FHT.h)
; ------------------
; FHTN     - FHT size	     
; FHTW     - window to apply
; FHTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
; FHTfsqrt - uses fast lookup sqrt() instead of accurate one
;
;
; 3. Files
; --------
; Whole FHT library consist of several files:
; 1. FHT_routines.s - main FHT algorithm routines
; 2. FHT_macros.inc - some higher level FHT related macros
; 3. FHT_tables.inc - FHT lookups
; 4. FHT_math.s     - FHT related arithmetics
; 5. FHT.c          - global FHTwarr[] array and some demo functions
; 6. FHT.h          - common FHT library header file, algorithm parameters
; 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
;
;
; 4. Performance
; --------------
; Measured execution times for FHTfsqrt=0, XTAL=16MHz:
;
; FHTprec=0, linear amplitude output:
; FFTN  input   transform  output  total    performance
; [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
; 32    0.096   0.293      0.606    0.995   32.16
; 64    0.188   0.714      1.201    2.103   30.43
; 128   0.371   1.692      2.406    4.469   28.64
; 256   0.771   3.919      4.810    9.500   26.95
; 512   1.539   8.972      9.626   20.137   25.43
;
; FHTprec=1, linear amplitude output:
; FFTN  input   transform  output  total    performance
; [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
; 32    0.096   0.327      0.606    1.029   32.16
; 64    0.188   0.798      1.201    2.187   30.43
; 128   0.371   1.876      2.406    4.653   28.64
; 256   0.771   4.325      4.810    9.906   26.95
; 512   1.539   9.876      9.626   21.041   25.43
;
; Output times for log scale spectrum less than 25% compared to linear scale:
; FFTN  output
; [-]   [ms]
; 32    0.139
; 64    0.273
; 128   0.541 
; 256   1.076
; 512   2.148
;
;
; 5. Licence
; ----------
; It's free for any projects you want but without any warranty since it's free and open.
;
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
;************************************************************************************************

#include <avr/io.h>

#include "FHT.h"

// general macros
#include "macros.inc"




;================================================================================================
;	32bit square root (526..542clk)
;	R16:R17 <= sqrt(R2:R3:R4:R5)
;	Author: ChaN (downloaded from ELM web (technical notes section))
;	        http://elm-chan.org/
;
;================================================================================================
;	Routine is made for AVRFHT/FFT only, it won't work as regular GCC function! 
;	Main reason for this is that it passes parameters in different registers.
;================================================================================================
	.global	FHT_sqrt32c_asm
	.func	FHT_sqrt32c_asm
	
FHT_sqrt32c_asm:	clr	R6
	clr	R7
	clr	R8
	clr	R9
	ldi	R16,1
	ldi	R17,0
	clr	R18
	clr	R19
	ldi	R20,16
	lsl	R2
	rol	R3
	rol	R4
	rol	R5
	rol	R6
	rol	R7
	rol	R8
	rol	R9
	lsl	R2
	rol	R3
	rol	R4
	rol	R5
	rol	R6
	rol	R7
	rol	R8
	rol	R9
	brpl	PC+(6)*2
	add	R6,R16
	adc	R7,R17
	adc	R8,R18
	adc	R9,R19
	rjmp	PC+(5)*2
	sub	R6,R16
	sbc	R7,R17
	sbc	R8,R18
	sbc	R9,R19
	lsl	R16
	rol	R17
	rol	R18
	andi	R16,0b11111000
	ori	R16,0b00000101
	sbrc	R9,7
	subi	R16,2
	dec	R20
	brne	PC-(34)*2
	lsr	R18
	ror	R17
	ror	R16
	lsr	R18
	ror	R17
	ror	R16
	ret
	.endfunc
	
;------------------------------------------------------------------------------------------------
;	GCC compatible wrapper for FHT_sqrt32c_asm() routine:
;
;	uint16_t FHT_sqrt32c(uint32_t x)
;------------------------------------------------------------------------------------------------	
	.global	FHT_sqrt32c
	.func	FHT_sqrt32c
	
FHT_sqrt32c:	movw	R30,R2
	movw	R26,R4
	movw	R0,R8
	movw	R2,R22	;low(x)
	movw	R4,R24	;high(x)
	movw	R22,R16
	movw	R24,R6
	;
	rcall	FHT_sqrt32c_asm	;do main sqrt()
	movw	R6,R24
	movw	R24,R16	;16bit output
	;
	movw	R16,R22
	movw	R8,R0
	movw	R4,R26
	movw	R2,R30
	clr	R1
	ret
	.endfunc

	


;================================================================================================
;	y=sqrt32(x) [db], where:
;	 - output: y=R16:R17
;	 - input: x=R2:R3:R4:R5
;	using R15 as zero reg
;
;	Worst precision is some 0.6% 
;
;	Author: Stanislav Maslan, s.maslan@seznam.cz
;
;================================================================================================
;	Routine is made for AVRFHT/FFT only, it won't work as regular GCC function! 
;	Main reason for this is that it passes parameters in different registers.
;================================================================================================
	.global	FHT_sqrt32f_asm
	.func	FHT_sqrt32f_asm

FHT_sqrt32f_asm:	clr	R15	;0
	tst	R5
	breq	sqrt32fn32
	;=== full 32-bit ===
	mov	R2,R4
	mov	R3,R5	;x>>=16
	rcall	sqrt32int	;s=sqrt(x)>>12
	ldi	R22,16
	mul	R20,R22
	movw	R16,R0
	mul	R21,R22
	add	R17,R0	;y=s<<12
	ret
	
sqrt32fn32:	tst	R4
	breq	sqrt32fn24
	;=== 24-bit ===
	mov	R2,R3
	mov	R3,R4	;x>>=8
	rcall	sqrt32int	;s=sqrt(x)>>8
	movw	R16,R20	;y=s<<8
	ret
	
sqrt32fn24:	tst	R3
	breq	sqrt32fn16
	;=== 16-bit ===
	rcall	sqrt32int	;s=sqrt(x)>>4
	ldi	R22,16
	mul	R21,R22
	movw	R16,R0
	mul	R20,R22
	add	R16,R1
	adc	R17,R15	;y=s<<4
	ret
	;=== 8-bit ===
sqrt32fn16:	ldi	R16,8
	cp	R2,R16
	brsh	sqrt32fn16lp
	;x<8 - switch to higher precision table
	ldi	R16,16*2
	mul	R2,R16
	movw	R30,R0
	subiw	R30,-(TSqrt256p1+1)
	rjmp	sqrt32fn16hp
	;x>8 - standard precision table
sqrt32fn16lp:	ldi	R16,8
	sub	R2,R16
	ldi	R16,2
	mul	R16,R2
	movw	R30,R0
	subiw	R30,-(TSqrt256p2+1)
sqrt32fn16hp:	lpm	R16,Z
	clr	R17	;y=sqrt(x)
	ret
	

;------------------------------------------------------------------------------------------------
;	return y=sqrt(x) from lookup table where: 
;	int part: x=R3
;	fraction: f=R2 [fx 0.8]
;	output: y=R20:R21 [fx 8.8]
;	using R15 as zero reg 
;------------------------------------------------------------------------------------------------
sqrt32int:	ldi	R16,8
	cp	R3,R16
	brsh	sqrt32intLP
	;x<8 - use higher precision lookup
	ldi	R22,16
	clr	R18
	mul	R2,R22
	movw	R16,R0
	mul	R3,R22
	addw	R17,R0	;(f:x)<<4
	mov	R2,R16	;f=((f:x)<<4)&0x000FF
	mov	R30,R17
	mov	R31,R18
	lslw	R30	;x=((f:x)>>4)&0x0FFFF
	subiw	R30,-(TSqrt256p1)	;Z=TSqrt256p1[x]	
	rjmp	sqrt32intHP
	;x>=8 - use standard precision loopup 
sqrt32intLP:	ldi	R16,8
	sub	R3,R16
	ldi	R16,2
	mul	R3,R16
	movw	R30,R0
	subiw	R30,-(TSqrt256p2)	;Z=TSqrt256p2[x]
sqrt32intHP:	lpmw	R20,Z+	;y1=sqrt(x)=*Z++ [fx 8.8]
	lpmw	R22,Z+	;y2=sqrt(x+1)=*Z++ [fx 8.8]
	subw	R22,R20	;dy=y2-y1 [fx 8.8]
	
	mul	R23,R2	
	addw	R20,R0
	mul	R22,R2
	add	R20,R1
	adc	R21,R15	;y=sqrt(y1+f*dy) [fx 8.8]
	ret	
	
	.type	TSqrt256p1,@object
	.size	TSqrt256p1,(16*8+1)*2
	;sqrt(x) [fx 8.8], for x=0,(1/16),(2/16)...(127/16),8 
TSqrt256p1:	.dw	0,64,91,111,128,143,157,169
	.dw	181,192,202,212,222,231,239,248
	.dw	256,264,272,279,286,293,300,307
	.dw	314,320,326,333,339,345,351,356
	.dw	362,368,373,379,384,389,395,400
	.dw	405,410,415,420,425,429,434,439
	.dw	443,448,453,457,462,466,470,475
	.dw	479,483,487,492,496,500,504,508
	.dw	512,516,520,524,528,532,535,539
	.dw	543,547,551,554,558,562,565,569
	.dw	572,576,580,583,587,590,594,597
	.dw	600,604,607,611,614,617,621,624
	.dw	627,630,634,637,640,643,646,650
	.dw	653,656,659,662,665,668,671,674
	.dw	677,680,683,686,689,692,695,698
	.dw	701,704,707,710,713,716,718,721
	.dw	724
	.type	TSqrt256p2,@object
	.size	TSqrt256p2,(256-8+1)*2
	;sqrt(x) [fx 8.8], for x=8...256
TSqrt256p2:	.dw	724,768,810,849,887,923,958,991
	.dw	1024,1056,1086,1116,1145,1173,1201,1228
	.dw	1254,1280,1305,1330,1355,1379,1402,1425
	.dw	1448,1471,1493,1515,1536,1557,1578,1599
	.dw	1619,1639,1659,1679,1698,1717,1736,1755
	.dw	1774,1792,1810,1828,1846,1864,1881,1899
	.dw	1916,1933,1950,1966,1983,1999,2016,2032
	.dw	2048,2064,2080,2095,2111,2126,2142,2157
	.dw	2172,2187,2202,2217,2232,2246,2261,2275
	.dw	2290,2304,2318,2332,2346,2360,2374,2388
	.dw	2401,2415,2429,2442,2455,2469,2482,2495
	.dw	2508,2521,2534,2547,2560,2573,2585,2598
	.dw	2611,2623,2636,2648,2660,2673,2685,2697
	.dw	2709,2721,2733,2745,2757,2769,2781,2793
	.dw	2804,2816,2828,2839,2851,2862,2874,2885
	.dw	2896,2908,2919,2930,2941,2952,2963,2974
	.dw	2985,2996,3007,3018,3029,3040,3051,3061
	.dw	3072,3083,3093,3104,3114,3125,3135,3146
	.dw	3156,3167,3177,3187,3197,3208,3218,3228
	.dw	3238,3248,3258,3268,3278,3288,3298,3308
	.dw	3318,3328,3338,3348,3357,3367,3377,3387
	.dw	3396,3406,3415,3425,3435,3444,3454,3463
	.dw	3473,3482,3491,3501,3510,3519,3529,3538
	.dw	3547,3556,3566,3575,3584,3593,3602,3611
	.dw	3620,3629,3638,3647,3656,3665,3674,3683
	.dw	3692,3701,3710,3719,3727,3736,3745,3754
	.dw	3762,3771,3780,3788,3797,3806,3814,3823
	.dw	3831,3840,3849,3857,3866,3874,3882,3891
	.dw	3899,3908,3916,3924,3933,3941,3949,3958
	.dw	3966,3974,3982,3991,3999,4007,4015,4023
	.dw	4031,4040,4048,4056,4064,4072,4080,4088
	.dw	4096
	.endfunc
	
;------------------------------------------------------------------------------------------------
;	GCC compatible wrapper for FHT_sqrt32f_asm() routine:
;
;	uint16_t FHT_sqrt32f(uint32_t x)
;------------------------------------------------------------------------------------------------	
	.global	FHT_sqrt32f
	.func	FHT_sqrt32f
	
FHT_sqrt32f:	movw	R26,R4
	movw	R4,R24	;high(x)
	movw	R24,R2
	movw	R2,R22	;low(x)
	mov	R24,R15
	pushw	R16
	;
	rcall	FHT_sqrt32f_asm	;do main sqrt()
	movw	R2,R22
	movw	R4,R24
	mov	R15,R24
	movw	R24,R16	;16bit output
	;
	popw	R16
	clr	R1	;zero
	ret
	.endfunc
	
	
		
	
	
	

;================================================================================================
;	This function is designed to calculate logarithmic amplitude spectrum 
;	from (re^2+im^2) term for FFT or (X(k)^2+X(-k)^2) for FHT. Result is 
;	returned directly in dB, singed fixed-point 8.8bit format.
;	The precision should be better than +/-3/256 (+/-0.012dB). 
;
;	Algorithm:
;	  y=20*log(sqrt(x)/xdb0)=10*log(x/xdb0^2), where xdb0 is amplitude 
;	  for 0dB output 
;	  
;	Author: Stanislav Maslan, s.maslan@seznam.cz
;================================================================================================
;	Routine is made for AVRFHT/FFT only, it won't work as regular GCC function! 
;	Main reason for this is that it passes parameters in different registers.
;================================================================================================
	.if	FHTW==wHANN
	;10*log10(x/x0db) data: x0dB^0.5=16384, xmax^0.5=32768 (+6.02dB), xmin^0.5=1 (-84.29dB)
	.equ	S32L10_db0,16384	;sqrt(x) for 0dB
	.equ	S32L10_dbmax,32768	;sqrt(x) for max dB
	.equ	S32L10_dbmax2,1073741824	;x for max dB
	.elif	FHTW==wHAMM
	;10*log10(x/x0db) data: x0dB^0.5=17695, xmax^0.5=35390 (+6.02dB), xmin^0.5=1 (-84.96dB)
	.equ	S32L10_db0,17695	;sqrt(x) for 0dB
	.equ	S32L10_dbmax,35390	;sqrt(x) for max dB
	.equ	S32L10_dbmax2,1252452100	;x for max dB
	.elif	FHTW==wBLAC
	;10*log10(x/x0db) data: x0dB^0.5=13762, xmax^0.5=27524 (+6.02dB), xmin^0.5=1 (-82.77dB)
	.equ	S32L10_db0,13762	;sqrt(x) for 0dB
	.equ	S32L10_dbmax,27524	;sqrt(x) for max dB
	.equ	S32L10_dbmax2,757570576	;x for max dB
	.elif	FHTW==wFLAT
	;10*log10(x/x0db) data: x0dB^0.5=7064, xmax^0.5=14128 (+6.02dB), xmin^0.5=1 (-76.98dB)
	.equ	S32L10_db0,7064	;sqrt(x) for 0dB
	.equ	S32L10_dbmax,14128	;sqrt(x) for max dB
	.equ	S32L10_dbmax2,199600384	;x for max dB
	.elif	FHTW==wRECT
	;10*log10(x/x0db) data: x0dB^0.5=32768, xmax^0.5=65535 (+6.02dB), xmin^0.5=1 (-90.31dB)
	.equ	S32L10_db0,32768	;sqrt(x) for 0dB
	.equ	S32L10_dbmax,65535	;sqrt(x) for max dB
	.equ	S32L10_dbmax2,4294836225	;x for max dB
	.endif
	.equ	S32L10_sc1m,8	;section 1 max
	.equ	S32L10_sc2m,48	;section 2 max
	.equ	S32L10_byc1,6165	;10*log10(2^8/db0^2) [fx 8.8]
	.equ	S32L10_byc2,12330	;10*log10(2^16/db0^2) [fx 8.8]
	.equ	S32L10_byc3,18495	;10*log10(2^24/db0^2) [fx 8.8]	

	.global	FHT_s32log10f_asm
	.func	FHT_s32log10f_asm

	;====== scaling x to 8.8 bit ======	
FHT_s32log10f_asm:	clr	R15
	;
	cp	R16,R15
	cpc	R17,R15
	cpc	R18,R15
	cpc	R19,R15
	brne	PC+(1+4)*2
	ldid	R16,1	;low limit to 1
	cpidh	R16,R20,S32L10_dbmax2
	brlo	PC+(1+4)*2
	ldid	R16,S32L10_dbmax2	;high limit to dbmax^2
	;
	tst	R19
	breq	s32log10fNb4
	;4 byte x
	lsl	R17
	movw	R16,R18
	adc	R16,R15
	adc	R17,R15	;x=round(x>>16)
	ldiw	R18,S32L10_byc3	;3 byte log offset
	rjmp	s32log10fSd
s32log10fNb4:	tst	R18
	breq	s32log10fNb3
	;3 byte x
	lsl	R16
	mov	R16,R17
	mov	R17,R18
	adc	R16,R15
	adc	R17,R15	;x=round(x>>8)
	ldiw	R18,S32L10_byc2	;2 byte log offset
	rjmp	s32log10fSd
s32log10fNb3:	tst	R17
	brne	s32log10fb2
	;1 byte x
	mov	R17,R16
	clr	R16	;x=(x<<8)
	ldiw	R18,0	;0 byte log offset
	rjmp	s32log10fSd
	;2 byte x
s32log10fb2:	ldiw	R18,S32L10_byc1	;1 byte log offset

	;====== scaling for different lookups ======
s32log10fSd:	cpi	R17,S32L10_sc1m
	brsh	s32log10fSdn1
	;lower lookup section
	dec	R17
	lslw	R16
	lslw	R16
	lslw	R16	;fixed:frac<<=3
	mov	R30,R17
	lsl	R30
	clr	R31
	subiw	R30,-(Ts32log10p1)	;Z=&Ts32log10p1[fixed]
	rjmp	s32log10fId	
s32log10fSdn1:	cpi	R17,S32L10_sc2m
	brlo	s32log10fSdn2
	;upper lookup section
	subi	R17,S32L10_sc2m
	lsrw	R16
	lsrw	R16
	adc	R16,R15
	adc	R17,R15	;fixed:frac=round(fixed:frac>>2)
	mov	R30,R17
	lsl	R30
	clr	R31
	subiw	R30,-(Ts32log10p3)	;Z=&Ts32log10p3[fixed]
	rjmp	s32log10fId
	;middle lookup section
s32log10fSdn2:	subi	R17,S32L10_sc1m
	mov	R30,R17
	lsl	R30
	clr	R31
	subiw	R30,-(Ts32log10p2)

	;====== main lookup interpolator ======	
s32log10fId:	lpmw	R20,Z+	;log[fixed]
	lpmw	R22,Z+	;log[fixed+1]
	subw	R22,R20	;dlog=log[fixed+1]-log[fixed]
	mulsu	R23,R16
	addw	R20,R0
	mul	R22,R16
	add	R20,R1
	adc	R21,R15
	addw	R20,R18
	movw	R16,R20	;out=logofs+(dlog*frac)>>8
	ret

	;====== lookups ======
	.type	Ts32log10p1,@object
	.size	Ts32log10p1,((S32L10_sc1m-1)<<3+1)*2
	.type	Ts32log10p2,@object
	.size	Ts32log10p2,((S32L10_sc2m-S32L10_sc1m)*8+1)*2
	.type	Ts32log10p3,@object
	.size	Ts32log10p3,((256-S32L10_sc2m)>>2+1)*2
	.if	FHTW==wHANN
	;10*log10(x/x0db) data: x0dB^0.5=16384, xmax^0.5=32768 (+6.02dB), xmin^0.5=1 (-84.29dB)
	;lookup part 1: x=<1,8>, step=0.125000, [fx 8.8dB]
Ts32log10p1:	.dw	-21576,-21445,-21328,-21223,-21126,-21037,-20955,-20878
	.dw	-20807,-20739,-20676,-20616,-20559,-20505,-20453,-20403
	.dw	-20356,-20311,-20267,-20225,-20185,-20146,-20108,-20072
	.dw	-20036,-20002,-19969,-19937,-19906,-19875,-19845,-19817
	.dw	-19788,-19761,-19734,-19708,-19682,-19657,-19633,-19609
	.dw	-19586,-19563,-19540,-19518,-19497,-19476,-19455,-19434
	.dw	-19414,-19395,-19375,-19356,-19338,-19319,-19301,-19283
	.dw	-19266
	;10*log10(x/x0db) data: x0dB^0.5=16384, xmax^0.5=32768 (+6.02dB), xmin^0.5=1 (-84.29dB)
	;lookup part 2: x=<8,48>, step=1.000000, [fx 8.8dB]
Ts32log10p2:	.dw	-19264,-19133,-19017,-18911,-18814,-18725,-18643,-18566
	.dw	-18495,-18427,-18364,-18304,-18247,-18193,-18141,-18092
	.dw	-18044,-17999,-17955,-17913,-17873,-17834,-17796,-17760
	.dw	-17725,-17690,-17657,-17625,-17594,-17563,-17533,-17505
	.dw	-17476,-17449,-17422,-17396,-17371,-17346,-17321,-17297
	.dw	-17274
	;10*log10(x/x0db) data: x0dB^0.5=16384, xmax^0.5=32768 (+6.02dB), xmin^0.5=1 (-84.29dB)
	;lookup part 3: x=<48,256>, step=4.000000, [fx 8.8dB]
Ts32log10p3:	.dw	-17273,-17184,-17102,-17025,-16953,-16886,-16823,-16763
	.dw	-16706,-16651,-16600,-16550,-16503,-16458,-16414,-16372
	.dw	-16332,-16293,-16255,-16219,-16183,-16149,-16116,-16084
	.dw	-16052,-16022,-15992,-15963,-15935,-15908,-15881,-15855
	.dw	-15829,-15804,-15780,-15756,-15733,-15710,-15687,-15665
	.dw	-15644,-15622,-15602,-15581,-15561,-15541,-15522,-15503
	.dw	-15484,-15466,-15448,-15430,-15413	

	.elif	FHTW==wHAMM
	;10*log10(x/x0db) data: x0dB^0.5=17695, xmax^0.5=35390 (+6.02dB), xmin^0.5=1 (-84.96dB)
	;lookup part 1: x=<1,8>, step=0.125000, [fx 8.8dB]
Ts32log10p1:	.dw	-21747,-21617,-21500,-21394,-21297,-21208,-21126,-21050
	.dw	-20978,-20911,-20847,-20787,-20730,-20676,-20624,-20575
	.dw	-20527,-20482,-20438,-20396,-20356,-20317,-20279,-20243
	.dw	-20208,-20173,-20140,-20108,-20077,-20046,-20017,-19988
	.dw	-19960,-19932,-19905,-19879,-19854,-19829,-19804,-19780
	.dw	-19757,-19734,-19711,-19689,-19668,-19647,-19626,-19606
	.dw	-19586,-19566,-19546,-19527,-19509,-19490,-19472,-19455
	.dw	-19437
	;10*log10(x/x0db) data: x0dB^0.5=17695, xmax^0.5=35390 (+6.02dB), xmin^0.5=1 (-84.96dB)
	;lookup part 2: x=<8,48>, step=1.000000, [fx 8.8dB]
Ts32log10p2:	.dw	-19435,-19305,-19188,-19082,-18985,-18897,-18814,-18738
	.dw	-18666,-18599,-18535,-18475,-18418,-18364,-18312,-18263
	.dw	-18215,-18170,-18126,-18085,-18044,-18005,-17967,-17931
	.dw	-17896,-17861,-17828,-17796,-17765,-17734,-17705,-17676
	.dw	-17648,-17620,-17593,-17567,-17542,-17517,-17492,-17468
	.dw	-17445
	;10*log10(x/x0db) data: x0dB^0.5=17695, xmax^0.5=35390 (+6.02dB), xmin^0.5=1 (-84.96dB)
	;lookup part 3: x=<48,256>, step=4.000000, [fx 8.8dB]
Ts32log10p3:	.dw	-17444,-17355,-17273,-17196,-17125,-17057,-16994,-16934
	.dw	-16877,-16823,-16771,-16721,-16674,-16629,-16585,-16543
	.dw	-16503,-16464,-16426,-16390,-16354,-16320,-16287,-16255
	.dw	-16223,-16193,-16163,-16135,-16106,-16079,-16052,-16026
	.dw	-16000,-15975,-15951,-15927,-15904,-15881,-15858,-15836
	.dw	-15815,-15794,-15773,-15752,-15732,-15713,-15693,-15674
	.dw	-15656,-15637,-15619,-15601,-15584
	
	.elif	FHTW==wBLAC
	;10*log10(x/x0db) data: x0dB^0.5=13762, xmax^0.5=27524 (+6.02dB), xmin^0.5=1 (-82.77dB)
	;lookup part 1: x=<1,8>, step=0.125000, [fx 8.8dB]
Ts32log10p1:	.dw	-21188,-21058,-20941,-20835,-20738,-20650,-20567,-20491
	.dw	-20419,-20352,-20288,-20228,-20171,-20117,-20065,-20016
	.dw	-19968,-19923,-19879,-19837,-19797,-19758,-19720,-19684
	.dw	-19649,-19614,-19581,-19549,-19518,-19487,-19458,-19429
	.dw	-19401,-19373,-19346,-19320,-19295,-19270,-19245,-19221
	.dw	-19198,-19175,-19153,-19131,-19109,-19088,-19067,-19047
	.dw	-19027,-19007,-18988,-18969,-18950,-18931,-18913,-18896
	.dw	-18878
	;10*log10(x/x0db) data: x0dB^0.5=13762, xmax^0.5=27524 (+6.02dB), xmin^0.5=1 (-82.77dB)
	;lookup part 2: x=<8,48>, step=1.000000, [fx 8.8dB]
Ts32log10p2:	.dw	-18876,-18746,-18629,-18523,-18426,-18338,-18255,-18179
	.dw	-18107,-18040,-17976,-17916,-17859,-17805,-17753,-17704
	.dw	-17656,-17611,-17568,-17526,-17485,-17446,-17408,-17372
	.dw	-17337,-17303,-17269,-17237,-17206,-17175,-17146,-17117
	.dw	-17089,-17061,-17034,-17008,-16983,-16958,-16933,-16909
	.dw	-16886
	;10*log10(x/x0db) data: x0dB^0.5=13762, xmax^0.5=27524 (+6.02dB), xmin^0.5=1 (-82.77dB)
	;lookup part 3: x=<48,256>, step=4.000000, [fx 8.8dB]
Ts32log10p3:	.dw	-16885,-16796,-16714,-16637,-16566,-16498,-16435,-16375
	.dw	-16318,-16264,-16212,-16163,-16115,-16070,-16026,-15984
	.dw	-15944,-15905,-15867,-15831,-15795,-15761,-15728,-15696
	.dw	-15665,-15634,-15604,-15576,-15547,-15520,-15493,-15467
	.dw	-15441,-15416,-15392,-15368,-15345,-15322,-15299,-15277
	.dw	-15256,-15235,-15214,-15193,-15173,-15154,-15134,-15115
	.dw	-15097,-15078,-15060,-15042,-15025
	
	.elif	FHTW==wFLAT
	;10*log10(x/x0db) data: x0dB^0.5=7064, xmax^0.5=14128 (+6.02dB), xmin^0.5=1 (-76.98dB)
	;lookup part 1: x=<1,8>, step=0.125000, [fx 8.8dB]
Ts32log10p1:	.dw	-19705,-19575,-19458,-19352,-19255,-19167,-19084,-19008
	.dw	-18936,-18869,-18805,-18745,-18688,-18634,-18582,-18533
	.dw	-18485,-18440,-18397,-18355,-18314,-18275,-18237,-18201
	.dw	-18166,-18132,-18098,-18066,-18035,-18004,-17975,-17946
	.dw	-17918,-17890,-17863,-17837,-17812,-17787,-17762,-17738
	.dw	-17715,-17692,-17670,-17648,-17626,-17605,-17584,-17564
	.dw	-17544,-17524,-17505,-17486,-17467,-17449,-17430,-17413
	.dw	-17395
	;10*log10(x/x0db) data: x0dB^0.5=7064, xmax^0.5=14128 (+6.02dB), xmin^0.5=1 (-76.98dB)
	;lookup part 2: x=<8,48>, step=1.000000, [fx 8.8dB]
Ts32log10p2:	.dw	-17393,-17263,-17146,-17040,-16944,-16855,-16772,-16696
	.dw	-16624,-16557,-16493,-16433,-16376,-16322,-16270,-16221
	.dw	-16174,-16128,-16085,-16043,-16002,-15963,-15926,-15889
	.dw	-15854,-15820,-15786,-15754,-15723,-15692,-15663,-15634
	.dw	-15606,-15578,-15552,-15525,-15500,-15475,-15450,-15427
	.dw	-15403
	;10*log10(x/x0db) data: x0dB^0.5=7064, xmax^0.5=14128 (+6.02dB), xmin^0.5=1 (-76.98dB)
	;lookup part 3: x=<48,256>, step=4.000000, [fx 8.8dB]
Ts32log10p3:	.dw	-15402,-15313,-15231,-15154,-15083,-15015,-14952,-14892
	.dw	-14835,-14781,-14729,-14680,-14632,-14587,-14543,-14501
	.dw	-14461,-14422,-14384,-14348,-14313,-14278,-14245,-14213
	.dw	-14182,-14151,-14122,-14093,-14065,-14037,-14010,-13984
	.dw	-13959,-13934,-13909,-13885,-13862,-13839,-13816,-13794
	.dw	-13773,-13752,-13731,-13710,-13690,-13671,-13651,-13632
	.dw	-13614,-13595,-13577,-13560,-13542
	
	.elif	FHTW==wRECT
	;10*log10(x/x0db) data: x0dB^0.5=32768, xmax^0.5=65535 (+6.02dB), xmin^0.5=1 (-90.31dB)
	;lookup part 1: x=<1,8>, step=0.125000, [fx 8.8dB]
Ts32log10p1:	.dw	-23117,-22987,-22870,-22764,-22667,-22579,-22496,-22420
	.dw	-22348,-22281,-22217,-22157,-22100,-22046,-21994,-21945
	.dw	-21897,-21852,-21808,-21767,-21726,-21687,-21649,-21613
	.dw	-21578,-21543,-21510,-21478,-21447,-21416,-21387,-21358
	.dw	-21330,-21302,-21275,-21249,-21224,-21199,-21174,-21150
	.dw	-21127,-21104,-21082,-21060,-21038,-21017,-20996,-20976
	.dw	-20956,-20936,-20917,-20898,-20879,-20861,-20842,-20825
	.dw	-20807
	;10*log10(x/x0db) data: x0dB^0.5=32768, xmax^0.5=65535 (+6.02dB), xmin^0.5=1 (-90.31dB)
	;lookup part 2: x=<8,48>, step=1.000000, [fx 8.8dB]
Ts32log10p2:	.dw	-20805,-20675,-20558,-20452,-20356,-20267,-20184,-20108
	.dw	-20036,-19969,-19905,-19845,-19788,-19734,-19682,-19633
	.dw	-19586,-19540,-19497,-19455,-19414,-19375,-19338,-19301
	.dw	-19266,-19232,-19198,-19166,-19135,-19104,-19075,-19046
	.dw	-19018,-18990,-18964,-18937,-18912,-18887,-18862,-18838
	.dw	-18815
	;10*log10(x/x0db) data: x0dB^0.5=32768, xmax^0.5=65535 (+6.02dB), xmin^0.5=1 (-90.31dB)
	;lookup part 3: x=<48,256>, step=4.000000, [fx 8.8dB]
Ts32log10p3:	.dw	-18814,-18725,-18643,-18566,-18495,-18427,-18364,-18304
	.dw	-18247,-18193,-18141,-18092,-18044,-17999,-17955,-17913
	.dw	-17873,-17834,-17796,-17760,-17725,-17690,-17657,-17625
	.dw	-17594,-17563,-17533,-17505,-17476,-17449,-17422,-17396
	.dw	-17371,-17346,-17321,-17297,-17274,-17251,-17228,-17206
	.dw	-17185,-17164,-17143,-17122,-17102,-17083,-17063,-17044
	.dw	-17026,-17007,-16989,-16971,-16954
	.endif
	.endfunc
	
;------------------------------------------------------------------------------------------------
;	GCC compatible wrapper for FHT_s32log10f_asm(x) routine:
;	  int16_t FHT_s32log10f(uint32_t x);
;------------------------------------------------------------------------------------------------
	.global	FHT_s32log10f
	.func	FHT_s32log10f
FHT_s32log10f:	movw	R26,R16
	movw	R16,R22
	movw	R18,R24	;function input
	mov	R25,R15
	rcall	FHT_s32log10f_asm
	mov	R15,R25
	movw	R24,R16	;function return
	;	
	movw	R16,R26
	clr	R1	;zero
	ret
	.endfunc


;------------------------------------------------------------------------------------------------
;	GCC compatible wrapper for FHT_s32log10f_asm(x^2) routine:
;	  int16_t FHT_log10f(uint16_t x);
;------------------------------------------------------------------------------------------------
	.global	FHT_log10f
	.func	FHT_log10f
FHT_log10f:	movw	R26,R16
	;
	clr	R21
	mul	R24,R24
	movw	R16,R0
	mul	R25,R25
	movw	R18,R0
	mul	R24,R25
	addw	R17,R0
	adc	R19,R21
	addw	R17,R0
	adc	R19,R21	;function input (x*=x)
	;
	mov	R25,R15
	rcall	FHT_s32log10f_asm
	mov	R15,R25
	movw	R24,R16	;function return
	;	
	movw	R16,R26
	clr	R1	;zero
	ret
	.endfunc
	

