;************************************************************************************************
; Fast Harltey Transform (FHT) for AVR - main FHT routines
;************************************************************************************************
; Fast FHT routines in 16-bit fixed point resolution. Implementation is based on Ullmann's paper
; from 1987 (good description and some examples of FFT and FHT):
;   http://sepwww.stanford.edu/theses/sep38/38_29.pdf  
;
; As expected the main transform needs only some 55% CPU time compared to FFT in the same
; resolution with the same optimalization techniques. 
; Overall performance is only 140% compared to FFT because amplitude spectrum output uses 
; accurate sqrt32() from ChanN. With log scale output the results are much better since it's 
; done using lookups (160% of FFT performance).
; But the main reason I made this routines is to save RAM because FHT uses real only work array.   
;
; All functions uses common global work array:
;   int16_t FHTwarr[FHTN]; 
; where FHTN is FHT size. Routines should work for FHTN={32,64,128,256,512}, lower resolutions 
; are also possible, but there are no lookups generated for them. 
;
;
; 1. Usage
; --------
; 1. call function FHT_input(int16_t *src)
;     - this function fills work array with bitreversed and windowed src data 
; 3. call FHT_do() to do the main FHT transform
;     - image is stored into the global work array FHTwarr[] already in order
; 4. call FHT_out_lin() or FHT_out_log() to create half of linear or log scale amplitude 
;    spectrum from FHT image. Results are stored again into FHTwarr[] array. 
;
;
; 2. Options (FHT.h)
; ------------------
; FHTN     - FHT size	     
; FHTW     - window to apply
; FHTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
; FHTfsqrt - uses fast lookup sqrt() instead of accurate one
;
;
; 3. Files
; --------
; Whole FHT library consist of several files:
; 1. FHT_routines.s - main FHT algorithm routines
; 2. FHT_macros.inc - some higher level FHT related macros
; 3. FHT_tables.inc - FHT lookups
; 4. FHT_math.s     - FHT related arithmetics
; 5. FHT.c          - global FHTwarr[] array and some demo functions
; 6. FHT.h          - common FHT library header file, algorithm parameters
; 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
;
;
; 4. Performance
; --------------
; Measured execution times for FHTfsqrt=0, XTAL=16MHz:
;
; FHTprec=0, linear amplitude output:
; FFTN  input   transform  output  total    performance
; [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
; 32    0.096   0.293      0.606    0.995   32.16
; 64    0.188   0.714      1.201    2.103   30.43
; 128   0.371   1.692      2.406    4.469   28.64
; 256   0.771   3.919      4.810    9.500   26.95
; 512   1.539   8.972      9.626   20.137   25.43
;
; FHTprec=1, linear amplitude output:
; FFTN  input   transform  output  total    performance
; [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
; 32    0.096   0.327      0.606    1.029   32.16
; 64    0.188   0.798      1.201    2.187   30.43
; 128   0.371   1.876      2.406    4.653   28.64
; 256   0.771   4.325      4.810    9.906   26.95
; 512   1.539   9.876      9.626   21.041   25.43
;
; Output times for log scale spectrum less than 25% compared to linear scale:
; FFTN  output
; [-]   [ms]
; 32    0.139
; 64    0.273
; 128   0.541 
; 256   1.076
; 512   2.148
;
;
; 5. Licence
; ----------
; It's free for any projects you want but without any warranty since it's free and open.
;
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
;************************************************************************************************
#include <avr/io.h>

#include "FHT.h"

// general macros
#include "macros.inc"

// FFT related macros
#include "FHT_macros.inc"

// FFT lookup tables
#include "FHT_tables.inc"


;================================================================================================
; Apply window and bitreversion to source data and stores result into global work array:
;   int16_t FHTwarr[FHTN];
; Source data can't be placed in FHTwarr[] array.
;
; Uses lookups:
;   TFHTbrv[] ... bit reversion indexes
;   TFHTwdw[] ... window function
;
; Algorithm:
;   FHTwarr[bit_rev(k)] = src[k]*window[k]
;
; AVRGCC function prototype:
;   void FHT_input(int16_t *src);
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
;================================================================================================
	.global	FHT_input
	.func	FHT_input

FHT_input:	push	R2
	pushd	R4
	pushw	R16
	pushw	R28
	
	clr	R2	;0
	.if	FHTW!=wRECT
	;=== for windows only ===
	ldiw	R30,TFHTwdw	;Z=FHT window
	ldiw	R20,TFHTbrv	;R20:R21=FHT bit reversion table
	movw	R28,R24	;Y=source data
	ldiw	R24,FHTN	;FHTN
FHTinpL1:	ldw	R16,Y+	;source=im{*Y++}
	lpmw	R18,Z+	;window_item=*Z++
		
	.ifdef	FHTWS
	fmuls	R17,R19	;for signed window
	.else
	fmulsu	R17,R19	;for unsigned window
	.endif
	movw	R6,R0
	fmul	R16,R18
	movw	R4,R0
	adc	R6,R2
	fmulsu	R17,R18
	sbc	R7,R2
	addw	R5,R0
	adc	R7,R2
	.ifdef	FHTWS
	fmulsu	R19,R16	;signed window
	sbc	R7,R2
	.else
	fmul	R19,R16	;unsigned window
	.endif
	addw	R5,R0
	adc	R7,R2	;res <= (un)signed(window_item)*signed(source) [fx 1.15]
		
	movw	R22,R30	;backup Z
	movw	R30,R20	;Z=R20:R21
	.if	FHTN>128
	lpmw	R26,Z+	;br=*Z++
	.else
	lpm	R26,Z+
	clr	R27	;br=*Z++
	.endif
	movw	R20,R30	;R20:R21=Z
	movw	R30,R22	;restore Z
	subiw	R26,-(FHTwarr)	;X=&FHTwarr[bit_rev(br)]
	stw	X+,R6	;re{*X}=res
	;new item
	dbnzwh	R24,FHTinpL1
	

	.else
	;=== if no window selected (rectangular) ===
	ldiw	R30,TFHTbrv	;Z=FHT bit reversion table
	movw	R28,R24	;Y=source data
	ldiw	R24,FHTN	;FHTN
FHTinpL1:	ldw	R16,Y+	;source=im{*Y++}
	.if	FHTN>128
	lpmw	R26,Z+	;br=*Z++
	.else
	lpm	R26,Z+
	clr	R27	;br=*Z++
	.endif
	subiw	R26,-(FHTwarr)	;X=&FHTwarr[bit_rev(br)]
	stw	X+,R16	;re{*X}=source
	;new item
	dbnzwh	R24,FHTinpL1
	.endif

	popw	R28
	popw	R16
	popd	R4
	pop	R2
	clr	R1
	ret
	.endfunc



	





;================================================================================================
; Fast Hartley Transform (FHT) algorithm for input sizes upto 512 points (N=2^M sizes only).
; It's based on Ullmann's paper from 1987:
;   http://sepwww.stanford.edu/theses/sep38/38_29.pdf
;
; The routine requires bit reversed input data, output data are in order.
; Uses lookup:
;   TFHTsico[] ... sine and cosine data
;  
; Uses global work array for both input and output:
;   int16_t FHTwarr[FHTN];
;
; AVRGCC function prototype:
;   void FHT_do(void);
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html 
;================================================================================================	
	.global	FHT_do
	.func	FHT_do
	
FHT_do:	pushd	R2
	pushd	R6
	pushd	R10
	pushd	R14
	pushw	R28
	
	;some constants
	clr	R2	;const: 0

	;init variables
	ldi	R16,low(FHTN/2)
	mov	R15,R16	;angf=FHTN/2
	clr	R14
	inc	R14	;n=1
	
	;=== for each FHT stage ===
FHTdoStage:
	;calculate butterfly step
	.if	FHTN>=512
	tst	R14
	brne	PC+(1+2+1+1)*2
	ldiw	R24,256*4
	movw	R0,R24
	rjmp	PC+(1+1+1+1)*2
	.endif
	ldi	R16,4
	mul	R14,R16
	movw	R24,R0	;bstep=2*n (*2 in bytes)
	
	;init primary butterfly pointers
	ldiw	R28,FHTwarr	;int16_t *pA=&FHTwarr[0]
	movw	R26,R28
	lsrw	R0
	addw	R26,R0	;int16_t *pB=&FHTwarr[n]
	movw	R30,R28
	addw	R30,R24
	sbiw	R30,2	;int16_t *pD=&FHTwarr[2*n-1]
	
	;=== zero frequency loop ===
	mov	R13,R15	;angf cycles
FHTdoZf:	;get and scale B
	ldw	R16,X+	;B=*pB++
	asrw	R16	;B>>=1 (fast version)
	.if	FHTprec
	tst	R17
	brpl	PC+(1+1+1)*2
	adc	R16,R2
	adc	R17,R2	;B/=2 (accurate version)
	.endif
	;get and scale A
	lddw	R18,Y+0	;A=*pA
	asrw	R18	;A>>=1 (fast version)
	.if	FHTprec
	tst	R19
	brpl	PC+(1+1+1)*2
	adc	R18,R2
	adc	R19,R2	;A/=2 (accurate version)
	.endif
	;do butterfly
	movw	R20,R18
	subw	R18,R16
	st	-X,R19
	st	-X,R18	;*-pB=(A-B)
	addw	R20,R16
	stdw	Y+0,R20	;*-pA=(A+B)
	;update buttefly position
	addw	R26,R24	;pB+=bstep
	addw	R28,R24	;pA+=bstep
	;next buttefly?
	dbnz	R13,FHTdoZf
	subiw	R26,(FHTN-1)*2	;pB-=(FHTN-1) (*2 in bytes)
	subiw	R28,(FHTN-1)*2	;pA-=(FHTN-1) (*2 in bytes)
	;
	
	.if	FHTN>=512
	tst	R14
	breq	PC+(1+1+1+2)*2
	.endif
	ldi	R16,2
	cp	R16,R14
	rjsh	FHTndoMf
	;=== n>2: do middle frequency band ===
	
	;init sin/cos pointer
	ldiw	R16,TFHTsico	;int16_t *sico=&TFHTsico[0]
	pushw	R16
	
	;init sico steps counter
	.if	FHTN>=512
	tst	R14
	brne	PC+(1+1+1+1)*2
	ldi	R16,(256>>1)-1
	mov	R13,R16
	rjmp	PC+(1+1+1+1)*2
	.endif
	mov	R13,R14
	lsr	R13
	dec	R13	;m=n/2-1
	
	;--- for each twidle angle ---
FHTdoMfAng:	;update sin/cos pointer
	movw	R4,R30
	popw	R30
	.if	FHTN>=512
	tst	R15
	brne	PC+(1+1+1)*2
	subi	R31,-4
	rjmp	PC+(1+1+1+2)*2
	.endif
	ldi	R16,4
	mul	R16,R15
	addw	R30,R0	;sico+=angf*2
	
	
	;get sin/cos values
	lpmw	R20,Z+	;dsin=sico[0]
	lpmw	R22,Z+	;dcos=sico[1]
	sbiw	R30,2*2
	pushw	R30	;store sico poitner
	movw	R30,R4
	
	pushw	R14	;store: anfg, n
	mov	R12,R15	;butterfly counter
	
	movw	R14,R30
	movw	R16,R24
	lsrw	R16
	subw	R14,R16	;int16_t *pC=pD-n
		
	
	;--- for each butterfly ---
FHTdoMfBut:	ldw	R16,X+	;B=*pB
	lddw	R18,Z+0	;D=*pD
	
	;vector rotation
	maddw	R16,R22,R18,R20,R4,R8,R10,R2	;E=B*dcos+D*dsin [fx 0.16]
	.if	FHTprec
	brpl	PC+(1+3)*2
	lsl	R5
	adc	R8,R2
	adc	R9,R2
	.endif
	msubw	R16,R20,R18,R22,R4,R6,R10,R2	;F=B*dsin-D*dcos [fx 0.16]
	.if	FHTprec
	brpl	PC+(1+3)*2
	lsl	R5
	adc	R6,R2
	adc	R7,R2
	.endif
	;do primary butterfly
	lddw	R16,Y+0	;A=*pA
	asrw	R16	;A>>=1 (fast version)
	.if	FHTprec
	tst	R17
	brpl	PC+(1+1+1)*2
	adc	R16,R2
	adc	R17,R2	;A/=2 (accurate version)
	.endif
	;
	movw	R18,R16
	addw	R16,R8	;(A+E)
	stdw	Y+0,R16	;*pA=A+E
	subw	R18,R8	;(A-E)
	st	-X,R19
	st	-X,R18	;*pB=A-E
	;do secondary butterfly
	movw	R4,R28
	movw	R28,R14	;load pC
	;
	lddw	R16,Y+0	;C=*pC
	asrw	R16	;C>>=1 (fast version)
	.if	FHTprec
	tst	R17
	brpl	PC+(1+1+1)*2
	adc	R16,R2
	adc	R17,R2	;C/=2 (accurate version)
	.endif
	;
	movw	R18,R16
	addw	R16,R6	;(C+F)
	stdw	Y+0,R16	;*pC=C+F
	subw	R18,R6	;(C-F)
	stdw	Z+0,R18	;*pD=C-F
	;
	movw	R14,R28
	movw	R28,R4	;reload pA
	;new butterfly positions
	addw	R28,R24	;pA+=bstep
	addw	R26,R24	;pB+=bstep
	addw	R14,R24	;pC+=bstep
	addw	R30,R24	;pD+=bstep
	;next cycle?
	djnz	R12,FHTdoMfBut
	;
	popw	R14	;restore: angf, n
	;
	subiw	R28,(FHTN-1)*2	;pA-=(FHTN-1) (*2 in bytes)
	subiw	R26,(FHTN-1)*2	;pB-=(FHTN-1) (*2 in bytes)
	subiw	R30,(FHTN+1)*2	;pD-=(FHTN+1) (*2 in bytes)
	;	
	djnz	R13,FHTdoMfAng
	popw	R0	;loose sico pointer
FHTndoMf:	
	
	.if	FHTN>=512
	tst	R14
	breq	PC+(1+1+1+2)*2
	.endif
	ldi	R16,1
	cp	R16,R14
	rjsh	FHTndoHf
	;=== n>1: Nyquist frequency ===
	
	;make C pointer from D pointer
	movw	R28,R30
	movw	R16,R24
	lsrw	R16
	subw	R28,R16	;int16_t *pC=pD-n
		
	mov	R13,R15	;anfg cycles	
	;get and scale D
FHTdoHf:	lddw	R16,Z+0	;D=*pD
	asrw	R16	;D>>=1 (fast version)
	.if	FHTprec
	tst	R17
	brpl	PC+(1+1+1)*2
	adc	R16,R2
	adc	R17,R2	;D/=2 (accurate version)
	.endif
	;get and scale C
	lddw	R18,Y+0	;C=*pC
	asrw	R18	;C>>=1 (fast version)
	.if	FHTprec
	tst	R19
	brpl	PC+(1+1+1)*2
	adc	R18,R2
	adc	R19,R2	;C/=2 (accurate version)
	.endif
	;do butterfly
	movw	R20,R18
	subw	R18,R16
	stdw	Z+0,R18	;*pD=(C-D)
	addw	R20,R16
	stdw	Y+0,R20	;*pC=(C+D)
	;update buttefly position
	addw	R28,R24	;pC+=bstep
	addw	R30,R24	;pD+=bstep
	;next buttefly?
	dbnz	R13,FHTdoHf
FHTndoHf:	
	lsl	R14	;n<<=1
	.if	FHTN>=512
	clc
	tst	R15
	brne	PC+(1+1)*2
	sec
	ror	R15
	.else
	lsr	R15	;angf>>=1
	.endif
	rjcc	FHTdoStage	;next stage?
	;
	popw	R28
	popd	R14
	popd	R10
	popd	R6
	popd	R2
	clr	R1
	ret
	.endfunc	
	

	



;================================================================================================
; Calculates linear scale amplitude spectrum from FHT image stored in global array:
;   int16_t FHTwarr[FHTN];
; It calculates only one half of the spectrum since it's symetrical.

; Algorithm: 
;   FHTwarr[k] = sqrt(FHTwarr[k]^2 + FHTwarr[-k]^2), for k=0..FHTN/2, where:
;     FHTwarr[-k] = FHTwarr[(FHTN-k) mod FHTN]
;
; AVRGCC function prototype:
;   void FHT_out_lin(void);
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
;================================================================================================
	.global	FHT_out_lin
	.func	FHT_out_lin
	;
FHT_out_lin:	pushd	R2
	pushd	R6
	pusht	R15
	pushw	R28
	;
	clr	R15
	ldiw	R28,FHTwarr	;int16_t *pA=FHTwarr[0]
	ldiw	R26,FHTwarr+FHTN*2	;int16_t *pB=FHTwarr[FHTN]
	ldsw	R18,FHTwarr	;zero frequency point
	ldi	R24,low(FHTN/2)	;half spectrum
FHTlinRep:	lddw	R16,Y+0	;A=*pA
	pwrsw	R16,R2,R15	;A*A
	pwrsw	R18,R6,R15	;B*B
	addd	R2,R6	;A*A+B*B
	;r=sqrt32(A*A+B*B)
	.if	FHTfsqrt
	call	FHT_sqrt32f_asm
	.else
	call	FHT_sqrt32c_asm
	.endif
	stw	Y+,R16	;*pA++=r
	ld	R19,-X
	ld	R18,-X	;B=*--pB
	dbnz	R24,FHTlinRep
	;
	popw	R28
	popt	R15
	popd	R2
	popd	R6
	clr	R1
	ret
	.endfunc
	
	




;================================================================================================
; Calculates log scale amplitude spectrum from FHT image stored in global array:
;   int16_t FHTwarr[FHTN];
; It calculates only one half of the spectrum since it's symetrical. Output is returned directly  
; in dB (fixed point 8.8).

; Algorithm: 
;   r[k] = sqrt(FHTwarr[k]^2 + FHTwarr[-k]^2), for k=0..FHTN/2, where:
;     FHTwarr[-k] = FHTwarr[(FHTN-k) mod FHTN]
;   FHTwarr[k] = 20*log10(r[k]/r0db), where r0db is fullscale sine input amplitude 
;
; AVRGCC function prototype:
;   void FHT_out_log(void);
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
;================================================================================================
	.global	FHT_out_log
	.func	FHT_out_log
	;
FHT_out_log:	pushd	R14
	pushw	R28
	;
	clr	R15
	ldiw	R28,FHTwarr	;int16_t *pA=FHTwarr[0]
	ldiw	R26,FHTwarr+FHTN*2	;int16_t *pB=FHTwarr[FHTN]
	ldsw	R20,FHTwarr	;zero frequency point
	ldi	R16,low(FHTN/2)	;half spectrum
	mov	R14,R16
FHTlogRep:	pwrsw	R20,R16,R15	;A*A
	lddw	R20,Y+0	;A=*pA
	pwrsw	R20,R22,R15	;B*B
	addd	R16,R22	;r=A*A+B*B
	call	FHT_s32log10f_asm	;r=10*log10(r/max) [dB] [fx 8.8]
	stw	Y+,R16	;*pA++=r
	ld	R21,-X
	ld	R20,-X	;B=*--pB
	dbnz	R14,FHTlogRep
	;
	popw	R28
	popd	R14
	clr	R1
	ret
	.endfunc	

