;************************************************************************************************
; Fast Harltey Transform (FHT) for AVR - FHT algorithm lookups
;************************************************************************************************
; Fast FHT routines in 16-bit fixed point resolution. Implementation is based on Ullmann's paper
; from 1987 (good description and some examples of FFT and FHT):
;   http://sepwww.stanford.edu/theses/sep38/38_29.pdf  
;
; As expected the main transform needs only some 55% CPU time compared to FFT in the same
; resolution with the same optimalization techniques. 
; Overall performance is only 140% compared to FFT because amplitude spectrum output uses 
; accurate sqrt32() from ChanN. With log scale output the results are much better since it's 
; done using lookups (160% of FFT performance).
; But the main reason I made this routines is to save RAM because FHT uses real only work array.   
;
; All functions uses common global work array:
;   int16_t FHTwarr[FHTN]; 
; where FHTN is FHT size. Routines should work for FHTN={32,64,128,256,512}, lower resolutions 
; are also possible, but there are no lookups generated for them. 
;
;
; 1. Usage
; --------
; 1. call function FHT_input(int16_t *src)
;     - this function fills work array with bitreversed and windowed src data 
; 3. call FHT_do() to do the main FHT transform
;     - image is stored into the global work array FHTwarr[] already in order
; 4. call FHT_out_lin() or FHT_out_log() to create half of linear or log scale amplitude 
;    spectrum from FHT image. Results are stored again into FHTwarr[] array. 
;
;
; 2. Options (FHT.h)
; ------------------
; FHTN     - FHT size	     
; FHTW     - window to apply
; FHTprec  - increases algorithm precision (lower algorithm noise), needs some more CPU power
; FHTfsqrt - uses fast lookup sqrt() instead of accurate one
;
;
; 3. Files
; --------
; Whole FHT library consist of several files:
; 1. FHT_routines.s - main FHT algorithm routines
; 2. FHT_macros.inc - some higher level FHT related macros
; 3. FHT_tables.inc - FHT lookups
; 4. FHT_math.s     - FHT related arithmetics
; 5. FHT.c          - global FHTwarr[] array and some demo functions
; 6. FHT.h          - common FHT library header file, algorithm parameters
; 7. macros.inc     - basic assembler macros, neccessary for all other assembler modules
;
;
; 4. Performance
; --------------
; Measured execution times for FHTfsqrt=0, XTAL=16MHz:
;
; FHTprec=0, linear amplitude output:
; FFTN  input   transform  output  total    performance
; [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
; 32    0.096   0.293      0.606    0.995   32.16
; 64    0.188   0.714      1.201    2.103   30.43
; 128   0.371   1.692      2.406    4.469   28.64
; 256   0.771   3.919      4.810    9.500   26.95
; 512   1.539   8.972      9.626   20.137   25.43
;
; FHTprec=1, linear amplitude output:
; FFTN  input   transform  output  total    performance
; [-]   [ms]    [ms]       [ms]    [ms]     [kp/s]
; 32    0.096   0.327      0.606    1.029   32.16
; 64    0.188   0.798      1.201    2.187   30.43
; 128   0.371   1.876      2.406    4.653   28.64
; 256   0.771   4.325      4.810    9.906   26.95
; 512   1.539   9.876      9.626   21.041   25.43
;
; Output times for log scale spectrum less than 25% compared to linear scale:
; FFTN  output
; [-]   [ms]
; 32    0.139
; 64    0.273
; 128   0.541 
; 256   1.076
; 512   2.148
;
;
; 5. Licence
; ----------
; It's free for any projects you want but without any warranty since it's free and open.
;
;
; (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
; http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/AVRFHT.html
;************************************************************************************************



;================================================================================================
;	FHT algorithm lookups for different FHT sizes
;
;	If you decide to add support for another windows that contains signed values
;	you'll have to add following definition: 
;	  .equ FHTWS,1
;================================================================================================

	;bit reversion table global
	.global TFHTbrv
	.type	TFHTbrv,@object
	.if	FHTN<256
	.size	TFHTbrv,FHTN
	.else
	.size	TFHTbrv,FHTN*2
	.endif
	
	;window table global
	.global TFHTwdw
	.type	TFHTwdw,@object
	.size	TFHTwdw,FHTN*2
	
	;sin/cos table
	.type	TFHTsico,@object
	.size	TFHTsico,FHTN
	

	.if	FHTN==32
	;====== Lookups for N=32 ======

	;Bit reversion for N=32, byte constants *2
TFHTbrv:	.db	0*2,16*2,8*2,24*2,4*2,20*2,12*2,28*2
	.db	2*2,18*2,10*2,26*2,6*2,22*2,14*2,30*2
	.db	1*2,17*2,9*2,25*2,5*2,21*2,13*2,29*2
	.db	3*2,19*2,11*2,27*2,7*2,23*2,15*2,31*2
	
	;sin(w), cos(w) values for w=<0,pi/2) for N=32 [fx 1.15]
TFHTsico:	.dw	     0, 32768,  6392, 32138, 12539, 30273, 18204, 27245
	.dw	 23170, 23170, 27245, 18204, 30273, 12539, 32138,  6392
	
	.if	FHTW==wHANN	
	;Hanning window for N=32 [fx 1.15]
TFHTwdw:	.dw	     0,   314,  1247,  2761,  4798,  7281, 10113, 13187
	.dw	 16383, 19579, 22653, 25485, 27968, 30005, 31519, 32452
	.dw	 32767, 32452, 31519, 30005, 27968, 25485, 22653, 19579
	.dw	 16383, 13187, 10113,  7281,  4798,  2761,  1247,   314

	.elif	FHTW==wHAMM
	;Hamming window for N=32 [fx 1.15]
TFHTwdw:	.dw	  2621,  2910,  3768,  5161,  7036,  9320, 11926, 14753
	.dw	 17694, 20634, 23462, 26068, 28352, 30226, 31619, 32477
	.dw	 32767, 32477, 31619, 30226, 28352, 26068, 23462, 20634
	.dw	 17694, 14753, 11926,  9320,  7036,  5161,  3768,  2910

	.elif	FHTW==wBLAC
	;Blackman window for N=32 [fx 1.15]
TFHTwdw:	.dw	     0,   115,   479,  1142,  2177,  3656,  5638,  8144
	.dw	 11140, 14536, 18178, 21861, 25347, 28387, 30752, 32252
	.dw	 32766, 32252, 30752, 28387, 25347, 21861, 18178, 14536
	.dw	 11140,  8144,  5638,  3656,  2177,  1142,   479,   115
	
	.elif	FHTW==wFLAT
	.equ	FHTWS,1
	;Flattop window for N=32 [fx 1.15]
TFHTwdw:	.dw	   -13,   -48,  -172,  -437,  -880, -1472, -2054, -2310
	.dw	 -1793,   -26,  3333,  8324, 14552, 21196, 27148, 31284
	.dw	 32767, 31284, 27148, 21196, 14552,  8324,  3333,   -26
	.dw	 -1793, -2310, -2054, -1472,  -880,  -437,  -172,   -48
	.endif
	
	.elif	FHTN==64
	;====== Lookups for N=64 ======
	
	;Bit reversion for N=64, byte constants *2
TFHTbrv:	.db	0*2,32*2,16*2,48*2,8*2,40*2,24*2,56*2
	.db	4*2,36*2,20*2,52*2,12*2,44*2,28*2,60*2
	.db	2*2,34*2,18*2,50*2,10*2,42*2,26*2,58*2
	.db	6*2,38*2,22*2,54*2,14*2,46*2,30*2,62*2
	.db	1*2,33*2,17*2,49*2,9*2,41*2,25*2,57*2
	.db	5*2,37*2,21*2,53*2,13*2,45*2,29*2,61*2
	.db	3*2,35*2,19*2,51*2,11*2,43*2,27*2,59*2
	.db	7*2,39*2,23*2,55*2,15*2,47*2,31*2,63*2

	;sin(w), cos(w) values for w=<0,pi/2) for N=64 [fx 1.15]
TFHTsico:	.dw	     0, 32768,  3211, 32610,  6392, 32138,  9512, 31357
	.dw	 12539, 30273, 15446, 28898, 18204, 27245, 20787, 25330
	.dw	 23170, 23170, 25330, 20787, 27245, 18204, 28898, 15446
	.dw	 30273, 12539, 31357,  9512, 32138,  6392, 32610,  3211

	.if	FHTW==wHANN
	;Hanning window for N=64 [fx 1.15]
TFHTwdw:	.dw	     0,    78,   314,   705,  1247,  1934,  2761,  3718
	.dw	  4798,  5989,  7281,  8660, 10113, 11627, 13187, 14777
	.dw	 16383, 17989, 19579, 21139, 22653, 24106, 25485, 26777
	.dw	 27968, 29048, 30005, 30832, 31519, 32061, 32452, 32688
	.dw	 32767, 32688, 32452, 32061, 31519, 30832, 30005, 29048
	.dw	 27968, 26777, 25485, 24106, 22653, 21139, 19579, 17989
	.dw	 16383, 14777, 13187, 11627, 10113,  8660,  7281,  5989
	.dw	  4798,  3718,  2761,  1934,  1247,   705,   314,    78

	.elif	FHTW==wHAMM
	;Hamming window for N=64 [fx 1.15]
TFHTwdw:	.dw	  2621,  2693,  2910,  3270,  3768,  4401,  5161,  6042
	.dw	  7036,  8132,  9320, 10588, 11926, 13318, 14753, 16216
	.dw	 17694, 19171, 20634, 22069, 23462, 24799, 26068, 27256
	.dw	 28352, 29345, 30226, 30987, 31619, 32117, 32477, 32694
	.dw	 32767, 32694, 32477, 32117, 31619, 30987, 30226, 29345
	.dw	 28352, 27256, 26068, 24799, 23462, 22069, 20634, 19171
	.dw	 17694, 16216, 14753, 13318, 11926, 10588,  9320,  8132
	.dw	  7036,  6042,  5161,  4401,  3768,  3270,  2910,  2693

	.elif	FHTW==wBLAC
	;Blackman window for N=64 [fx 1.15]
TFHTwdw:	.dw	     0,    28,   115,   263,   479,   769,  1142,  1608
	.dw	  2177,  2857,  3656,  4582,  5638,  6826,  8144,  9585
	.dw	 11140, 12797, 14536, 16338, 18178, 20028, 21861, 23644
	.dw	 25347, 26938, 28387, 29667, 30752, 31619, 32252, 32637
	.dw	 32766, 32637, 32252, 31619, 30752, 29667, 28387, 26938
	.dw	 25347, 23644, 21861, 20028, 18178, 16338, 14536, 12797
	.dw	 11140,  9585,  8144,  6826,  5638,  4582,  3656,  2857
	.dw	  2177,  1608,  1142,   769,   479,   263,   115,    28
	
	.elif	FHTW==wFLAT
	.equ	FHTWS,1
	;Flattop window for N=64 [fx 1.15]
TFHTwdw:	.dw	   -13,   -22,   -48,   -96,  -172,  -284,  -437,  -636
	.dw	  -880, -1164, -1472, -1780, -2054, -2248, -2310, -2179
	.dw	 -1793, -1093,   -26,  1442,  3333,  5639,  8324, 11326
	.dw	 14552, 17887, 21196, 24332, 27148, 29506, 31284, 32391
	.dw	 32767, 32391, 31284, 29506, 27148, 24332, 21196, 17887
	.dw	 14552, 11326,  8324,  5639,  3333,  1442,   -26, -1093
	.dw	 -1793, -2179, -2310, -2248, -2054, -1780, -1472, -1164
	.dw	  -880,  -636,  -437,  -284,  -172,   -96,   -48,   -22
	.endif
	
	.elif	FHTN==128
	;====== Lookups for N=128 ======
	
	;Bit reversion for N=128, byte constants *2
TFHTbrv:	.db	0*2,64*2,32*2,96*2,16*2,80*2,48*2,112*2
	.db	8*2,72*2,40*2,104*2,24*2,88*2,56*2,120*2
	.db	4*2,68*2,36*2,100*2,20*2,84*2,52*2,116*2
	.db	12*2,76*2,44*2,108*2,28*2,92*2,60*2,124*2
	.db	2*2,66*2,34*2,98*2,18*2,82*2,50*2,114*2
	.db	10*2,74*2,42*2,106*2,26*2,90*2,58*2,122*2
	.db	6*2,70*2,38*2,102*2,22*2,86*2,54*2,118*2
	.db	14*2,78*2,46*2,110*2,30*2,94*2,62*2,126*2
	.db	1*2,65*2,33*2,97*2,17*2,81*2,49*2,113*2
	.db	9*2,73*2,41*2,105*2,25*2,89*2,57*2,121*2
	.db	5*2,69*2,37*2,101*2,21*2,85*2,53*2,117*2
	.db	13*2,77*2,45*2,109*2,29*2,93*2,61*2,125*2
	.db	3*2,67*2,35*2,99*2,19*2,83*2,51*2,115*2
	.db	11*2,75*2,43*2,107*2,27*2,91*2,59*2,123*2
	.db	7*2,71*2,39*2,103*2,23*2,87*2,55*2,119*2
	.db	15*2,79*2,47*2,111*2,31*2,95*2,63*2,127*2

	;sin(w), cos(w) values for w=<0,pi/2) for N=128 [fx 1.15]
TFHTsico:	.dw	     0, 32768,  1607, 32728,  3211, 32610,  4808, 32413
	.dw	  6392, 32138,  7961, 31785,  9512, 31357, 11039, 30852
	.dw	 12539, 30273, 14010, 29621, 15446, 28898, 16846, 28106
	.dw	 18204, 27245, 19519, 26319, 20787, 25330, 22005, 24279
	.dw	 23170, 23170, 24279, 22005, 25330, 20787, 26319, 19519
	.dw	 27245, 18204, 28106, 16846, 28898, 15446, 29621, 14010
	.dw	 30273, 12539, 30852, 11039, 31357,  9512, 31785,  7961
	.dw	 32138,  6392, 32413,  4808, 32610,  3211, 32728,  1607

	.if	FHTW==wHANN
	;Hanning window for N=128 [fx 1.15]
TFHTwdw:	.dw	     0,    19,    78,   177,   314,   490,   705,   957
	.dw	  1247,  1572,  1934,  2330,  2761,  3224,  3718,  4244
	.dw	  4798,  5381,  5989,  6623,  7281,  7960,  8660,  9378
	.dw	 10113, 10864, 11627, 12402, 13187, 13979, 14777, 15579
	.dw	 16383, 17187, 17989, 18787, 19579, 20364, 21139, 21902
	.dw	 22653, 23388, 24106, 24806, 25485, 26143, 26777, 27385
	.dw	 27968, 28522, 29048, 29542, 30005, 30436, 30832, 31194
	.dw	 31519, 31809, 32061, 32276, 32452, 32589, 32688, 32747
	.dw	 32767, 32747, 32688, 32589, 32452, 32276, 32061, 31809
	.dw	 31519, 31194, 30832, 30436, 30005, 29542, 29048, 28522
	.dw	 27968, 27385, 26777, 26143, 25485, 24806, 24106, 23388
	.dw	 22653, 21902, 21139, 20364, 19579, 18787, 17989, 17187
	.dw	 16383, 15579, 14777, 13979, 13187, 12402, 11627, 10864
	.dw	 10113,  9378,  8660,  7960,  7281,  6623,  5989,  5381
	.dw	  4798,  4244,  3718,  3224,  2761,  2330,  1934,  1572
	.dw	  1247,   957,   705,   490,   314,   177,    78,    19

	.elif	FHTW==wHAMM
	;Hamming window for N=128 [fx 1.15]
TFHTwdw:	.dw	  2621,  2639,  2693,  2784,  2910,  3073,  3270,  3502
	.dw	  3768,  4068,  4401,  4765,  5161,  5587,  6042,  6525
	.dw	  7036,  7571,  8132,  8715,  9320,  9945, 10588, 11249
	.dw	 11926, 12616, 13318, 14031, 14753, 15482, 16216, 16954
	.dw	 17694, 18433, 19171, 19905, 20634, 21356, 22069, 22772
	.dw	 23462, 24138, 24799, 25443, 26068, 26673, 27256, 27816
	.dw	 28352, 28862, 29345, 29800, 30226, 30622, 30987, 31319
	.dw	 31619, 31885, 32117, 32315, 32477, 32603, 32694, 32748
	.dw	 32767, 32748, 32694, 32603, 32477, 32315, 32117, 31885
	.dw	 31619, 31319, 30987, 30622, 30226, 29800, 29345, 28862
	.dw	 28352, 27816, 27256, 26673, 26068, 25443, 24799, 24138
	.dw	 23462, 22772, 22069, 21356, 20634, 19905, 19171, 18433
	.dw	 17694, 16954, 16216, 15482, 14753, 14031, 13318, 12616
	.dw	 11926, 11249, 10588,  9945,  9320,  8715,  8132,  7571
	.dw	  7036,  6525,  6042,  5587,  5161,  4765,  4401,  4068
	.dw	  3768,  3502,  3270,  3073,  2910,  2784,  2693,  2639

	.elif	FHTW==wBLAC
	;Blackman window for N=128 [fx 1.15]
TFHTwdw:	.dw	     0,     7,    28,    64,   115,   181,   263,   362
	.dw	   479,   614,   769,   945,  1142,  1363,  1608,  1879
	.dw	  2177,  2502,  2857,  3241,  3656,  4103,  4582,  5094
	.dw	  5638,  6216,  6826,  7469,  8144,  8849,  9585, 10349
	.dw	 11140, 11957, 12797, 13657, 14536, 15431, 16338, 17255
	.dw	 18178, 19104, 20028, 20949, 21861, 22760, 23644, 24507
	.dw	 25347, 26158, 26938, 27682, 28387, 29050, 29667, 30235
	.dw	 30752, 31214, 31619, 31966, 32252, 32476, 32637, 32734
	.dw	 32766, 32734, 32637, 32476, 32252, 31966, 31619, 31214
	.dw	 30752, 30235, 29667, 29050, 28387, 27682, 26938, 26158
	.dw	 25347, 24507, 23644, 22760, 21861, 20949, 20028, 19104
	.dw	 18178, 17255, 16338, 15431, 14536, 13657, 12797, 11957
	.dw	 11140, 10349,  9585,  8849,  8144,  7469,  6826,  6216
	.dw	  5638,  5094,  4582,  4103,  3656,  3241,  2857,  2502
	.dw	  2177,  1879,  1608,  1363,  1142,   945,   769,   614
	.dw	   479,   362,   263,   181,   115,    64,    28,     7
	
	.elif	FHTW==wFLAT
	.equ	FHTWS,1
	;Flattop window for N=128 [fx 1.15]
TFHTwdw:	.dw	   -13,   -15,   -22,   -32,   -48,   -69,   -96,  -130
	.dw	  -172,  -223,  -284,  -355,  -437,  -530,  -636,  -752
	.dw	  -880, -1018, -1164, -1316, -1472, -1628, -1780, -1924
	.dw	 -2054, -2164, -2248, -2299, -2310, -2272, -2179, -2022
	.dw	 -1793, -1486, -1093,  -608,   -26,   655,  1442,  2335
	.dw	  3333,  4436,  5639,  6937,  8324,  9791, 11326, 12918
	.dw	 14552, 16214, 17887, 19554, 21196, 22795, 24332, 25789
	.dw	 27148, 28392, 29506, 30474, 31284, 31926, 32391, 32672
	.dw	 32767, 32672, 32391, 31926, 31284, 30474, 29506, 28392
	.dw	 27148, 25789, 24332, 22795, 21196, 19554, 17887, 16214
	.dw	 14552, 12918, 11326,  9791,  8324,  6937,  5639,  4436
	.dw	  3333,  2335,  1442,   655,   -26,  -608, -1093, -1486
	.dw	 -1793, -2022, -2179, -2272, -2310, -2299, -2248, -2164
	.dw	 -2054, -1924, -1780, -1628, -1472, -1316, -1164, -1018
	.dw	  -880,  -752,  -636,  -530,  -437,  -355,  -284,  -223
	.dw	  -172,  -130,   -96,   -69,   -48,   -32,   -22,   -15
	.endif
	
	
	.elif	FHTN==256
	;====== Lookups for N=256 ======
	
	;Bit reversion for N=256, byte constants *2
TFHTbrv:	.dw	0*2,128*2,64*2,192*2,32*2,160*2,96*2,224*2
	.dw	16*2,144*2,80*2,208*2,48*2,176*2,112*2,240*2
	.dw	8*2,136*2,72*2,200*2,40*2,168*2,104*2,232*2
	.dw	24*2,152*2,88*2,216*2,56*2,184*2,120*2,248*2
	.dw	4*2,132*2,68*2,196*2,36*2,164*2,100*2,228*2
	.dw	20*2,148*2,84*2,212*2,52*2,180*2,116*2,244*2
	.dw	12*2,140*2,76*2,204*2,44*2,172*2,108*2,236*2
	.dw	28*2,156*2,92*2,220*2,60*2,188*2,124*2,252*2
	.dw	2*2,130*2,66*2,194*2,34*2,162*2,98*2,226*2
	.dw	18*2,146*2,82*2,210*2,50*2,178*2,114*2,242*2
	.dw	10*2,138*2,74*2,202*2,42*2,170*2,106*2,234*2
	.dw	26*2,154*2,90*2,218*2,58*2,186*2,122*2,250*2
	.dw	6*2,134*2,70*2,198*2,38*2,166*2,102*2,230*2
	.dw	22*2,150*2,86*2,214*2,54*2,182*2,118*2,246*2
	.dw	14*2,142*2,78*2,206*2,46*2,174*2,110*2,238*2
	.dw	30*2,158*2,94*2,222*2,62*2,190*2,126*2,254*2
	.dw	1*2,129*2,65*2,193*2,33*2,161*2,97*2,225*2
	.dw	17*2,145*2,81*2,209*2,49*2,177*2,113*2,241*2
	.dw	9*2,137*2,73*2,201*2,41*2,169*2,105*2,233*2
	.dw	25*2,153*2,89*2,217*2,57*2,185*2,121*2,249*2
	.dw	5*2,133*2,69*2,197*2,37*2,165*2,101*2,229*2
	.dw	21*2,149*2,85*2,213*2,53*2,181*2,117*2,245*2
	.dw	13*2,141*2,77*2,205*2,45*2,173*2,109*2,237*2
	.dw	29*2,157*2,93*2,221*2,61*2,189*2,125*2,253*2
	.dw	3*2,131*2,67*2,195*2,35*2,163*2,99*2,227*2
	.dw	19*2,147*2,83*2,211*2,51*2,179*2,115*2,243*2
	.dw	11*2,139*2,75*2,203*2,43*2,171*2,107*2,235*2
	.dw	27*2,155*2,91*2,219*2,59*2,187*2,123*2,251*2
	.dw	7*2,135*2,71*2,199*2,39*2,167*2,103*2,231*2
	.dw	23*2,151*2,87*2,215*2,55*2,183*2,119*2,247*2
	.dw	15*2,143*2,79*2,207*2,47*2,175*2,111*2,239*2
	.dw	31*2,159*2,95*2,223*2,63*2,191*2,127*2,255*2

	;sin(w), cos(w) values for w=<0,pi/2) for N=256 [fx 1.15]
TFHTsico:	.dw	     0, 32768,   804, 32758,  1607, 32728,  2410, 32679
	.dw	  3211, 32610,  4011, 32521,  4808, 32413,  5602, 32285
	.dw	  6392, 32138,  7179, 31971,  7961, 31785,  8739, 31581
	.dw	  9512, 31357, 10278, 31114, 11039, 30852, 11793, 30572
	.dw	 12539, 30273, 13278, 29956, 14010, 29621, 14732, 29269
	.dw	 15446, 28898, 16151, 28511, 16846, 28106, 17530, 27684
	.dw	 18204, 27245, 18868, 26790, 19519, 26319, 20159, 25832
	.dw	 20787, 25330, 21403, 24812, 22005, 24279, 22594, 23732
	.dw	 23170, 23170, 23732, 22594, 24279, 22005, 24812, 21403
	.dw	 25330, 20787, 25832, 20159, 26319, 19519, 26790, 18868
	.dw	 27245, 18204, 27684, 17530, 28106, 16846, 28511, 16151
	.dw	 28898, 15446, 29269, 14732, 29621, 14010, 29956, 13278
	.dw	 30273, 12539, 30572, 11793, 30852, 11039, 31114, 10278
	.dw	 31357,  9512, 31581,  8739, 31785,  7961, 31971,  7179
	.dw	 32138,  6392, 32285,  5602, 32413,  4808, 32521,  4011
	.dw	 32610,  3211, 32679,  2410, 32728,  1607, 32758,   804

	.if	FHTW==wHANN
	;Hanning window for N=256 [fx 1.15]
TFHTwdw:	.dw	     0,     4,    19,    44,    78,   123,   177,   241
	.dw	   314,   398,   490,   593,   705,   826,   957,  1097
	.dw	  1247,  1405,  1572,  1749,  1934,  2128,  2330,  2541
	.dw	  2761,  2988,  3224,  3467,  3718,  3977,  4244,  4517
	.dw	  4798,  5086,  5381,  5682,  5989,  6303,  6623,  6949
	.dw	  7281,  7618,  7960,  8308,  8660,  9017,  9378,  9744
	.dw	 10113, 10487, 10864, 11244, 11627, 12013, 12402, 12793
	.dw	 13187, 13582, 13979, 14377, 14777, 15178, 15579, 15981
	.dw	 16383, 16785, 17187, 17588, 17989, 18389, 18787, 19184
	.dw	 19579, 19973, 20364, 20753, 21139, 21522, 21902, 22279
	.dw	 22653, 23022, 23388, 23749, 24106, 24458, 24806, 25148
	.dw	 25485, 25817, 26143, 26463, 26777, 27084, 27385, 27680
	.dw	 27968, 28249, 28522, 28789, 29048, 29299, 29542, 29778
	.dw	 30005, 30225, 30436, 30638, 30832, 31017, 31194, 31361
	.dw	 31519, 31669, 31809, 31940, 32061, 32173, 32276, 32368
	.dw	 32452, 32525, 32589, 32643, 32688, 32722, 32747, 32762
	.dw	 32767, 32762, 32747, 32722, 32688, 32643, 32589, 32525
	.dw	 32452, 32368, 32276, 32173, 32061, 31940, 31809, 31669
	.dw	 31519, 31361, 31194, 31017, 30832, 30638, 30436, 30225
	.dw	 30005, 29778, 29542, 29299, 29048, 28789, 28522, 28249
	.dw	 27968, 27680, 27385, 27084, 26777, 26463, 26143, 25817
	.dw	 25485, 25148, 24806, 24458, 24106, 23749, 23388, 23022
	.dw	 22653, 22279, 21902, 21522, 21139, 20753, 20364, 19973
	.dw	 19579, 19184, 18787, 18389, 17989, 17588, 17187, 16785
	.dw	 16383, 15981, 15579, 15178, 14777, 14377, 13979, 13582
	.dw	 13187, 12793, 12402, 12013, 11627, 11244, 10864, 10487
	.dw	 10113,  9744,  9378,  9017,  8660,  8308,  7960,  7618
	.dw	  7281,  6949,  6623,  6303,  5989,  5682,  5381,  5086
	.dw	  4798,  4517,  4244,  3977,  3718,  3467,  3224,  2988
	.dw	  2761,  2541,  2330,  2128,  1934,  1749,  1572,  1405
	.dw	  1247,  1097,   957,   826,   705,   593,   490,   398
	.dw	   314,   241,   177,   123,    78,    44,    19,     4

	.elif	FHTW==wHAMM
	;Hamming window for N=256 [fx 1.15]
TFHTwdw:	.dw	  2621,  2625,  2639,  2662,  2693,  2734,  2784,  2843
	.dw	  2910,  2987,  3073,  3167,  3270,  3382,  3502,  3631
	.dw	  3768,  3914,  4068,  4230,  4401,  4579,  4765,  4959
	.dw	  5161,  5370,  5587,  5811,  6042,  6280,  6525,  6777
	.dw	  7036,  7300,  7571,  7849,  8132,  8420,  8715,  9015
	.dw	  9320,  9630,  9945, 10264, 10588, 10917, 11249, 11586
	.dw	 11926, 12269, 12616, 12966, 13318, 13674, 14031, 14391
	.dw	 14753, 15117, 15482, 15849, 16216, 16585, 16954, 17324
	.dw	 17694, 18064, 18433, 18803, 19171, 19539, 19905, 20271
	.dw	 20634, 20996, 21356, 21714, 22069, 22422, 22772, 23118
	.dw	 23462, 23802, 24138, 24471, 24799, 25123, 25443, 25758
	.dw	 26068, 26373, 26673, 26967, 27256, 27539, 27816, 28087
	.dw	 28352, 28610, 28862, 29107, 29345, 29576, 29800, 30017
	.dw	 30226, 30428, 30622, 30808, 30987, 31157, 31319, 31473
	.dw	 31619, 31757, 31885, 32006, 32117, 32221, 32315, 32400
	.dw	 32477, 32545, 32603, 32653, 32694, 32726, 32748, 32762
	.dw	 32767, 32762, 32748, 32726, 32694, 32653, 32603, 32545
	.dw	 32477, 32400, 32315, 32221, 32117, 32006, 31885, 31757
	.dw	 31619, 31473, 31319, 31157, 30987, 30808, 30622, 30428
	.dw	 30226, 30017, 29800, 29576, 29345, 29107, 28862, 28610
	.dw	 28352, 28087, 27816, 27539, 27256, 26967, 26673, 26373
	.dw	 26068, 25758, 25443, 25123, 24799, 24471, 24138, 23802
	.dw	 23462, 23118, 22772, 22422, 22069, 21714, 21356, 20996
	.dw	 20634, 20271, 19905, 19539, 19171, 18803, 18433, 18064
	.dw	 17694, 17324, 16954, 16585, 16216, 15849, 15482, 15117
	.dw	 14753, 14391, 14031, 13674, 13318, 12966, 12616, 12269
	.dw	 11926, 11586, 11249, 10917, 10588, 10264,  9945,  9630
	.dw	  9320,  9015,  8715,  8420,  8132,  7849,  7571,  7300
	.dw	  7036,  6777,  6525,  6280,  6042,  5811,  5587,  5370
	.dw	  5161,  4959,  4765,  4579,  4401,  4230,  4068,  3914
	.dw	  3768,  3631,  3502,  3382,  3270,  3167,  3073,  2987
	.dw	  2910,  2843,  2784,  2734,  2693,  2662,  2639,  2625

	.elif	FHTW==wBLAC
	;Blackman window for N=256 [fx 1.15]
TFHTwdw:	.dw	     0,     1,     7,    16,    28,    44,    64,    87
	.dw	   115,   146,   181,   220,   263,   311,   362,   418
	.dw	   479,   544,   614,   689,   769,   854,   945,  1041
	.dw	  1142,  1250,  1363,  1483,  1608,  1741,  1879,  2025
	.dw	  2177,  2336,  2502,  2676,  2857,  3045,  3241,  3445
	.dw	  3656,  3876,  4103,  4339,  4582,  4834,  5094,  5362
	.dw	  5638,  5923,  6216,  6517,  6826,  7144,  7469,  7802
	.dw	  8144,  8493,  8849,  9213,  9585,  9963, 10349, 10741
	.dw	 11140, 11546, 11957, 12374, 12797, 13224, 13657, 14094
	.dw	 14536, 14982, 15431, 15883, 16338, 16795, 17255, 17716
	.dw	 18178, 18641, 19104, 19566, 20028, 20489, 20949, 21406
	.dw	 21861, 22312, 22760, 23204, 23644, 24078, 24507, 24930
	.dw	 25347, 25756, 26158, 26552, 26938, 27314, 27682, 28040
	.dw	 28387, 28724, 29050, 29364, 29667, 29957, 30235, 30500
	.dw	 30752, 30990, 31214, 31424, 31619, 31800, 31966, 32117
	.dw	 32252, 32372, 32476, 32565, 32637, 32694, 32734, 32758
	.dw	 32766, 32758, 32734, 32694, 32637, 32565, 32476, 32372
	.dw	 32252, 32117, 31966, 31800, 31619, 31424, 31214, 30990
	.dw	 30752, 30500, 30235, 29957, 29667, 29364, 29050, 28724
	.dw	 28387, 28040, 27682, 27314, 26938, 26552, 26158, 25756
	.dw	 25347, 24930, 24507, 24078, 23644, 23204, 22760, 22312
	.dw	 21861, 21406, 20949, 20489, 20028, 19566, 19104, 18641
	.dw	 18178, 17716, 17255, 16795, 16338, 15883, 15431, 14982
	.dw	 14536, 14094, 13657, 13224, 12797, 12374, 11957, 11546
	.dw	 11140, 10741, 10349,  9963,  9585,  9213,  8849,  8493
	.dw	  8144,  7802,  7469,  7144,  6826,  6517,  6216,  5923
	.dw	  5638,  5362,  5094,  4834,  4582,  4339,  4103,  3876
	.dw	  3656,  3445,  3241,  3045,  2857,  2676,  2502,  2336
	.dw	  2177,  2025,  1879,  1741,  1608,  1483,  1363,  1250
	.dw	  1142,  1041,   945,   854,   769,   689,   614,   544
	.dw	   479,   418,   362,   311,   263,   220,   181,   146
	.dw	   115,    87,    64,    44,    28,    16,     7,     1
	
	.elif	FHTW==wFLAT
	.equ	FHTWS,1
	;Flattop window for N=256 [fx 1.15]
TFHTwdw:	.dw	   -13,   -14,   -15,   -18,   -22,   -26,   -32,   -39
	.dw	   -48,   -57,   -69,   -81,   -96,  -112,  -130,  -150
	.dw	  -172,  -196,  -223,  -252,  -284,  -318,  -355,  -394
	.dw	  -437,  -482,  -530,  -581,  -636,  -692,  -752,  -815
	.dw	  -880,  -948, -1018, -1090, -1164, -1239, -1316, -1394
	.dw	 -1472, -1550, -1628, -1705, -1780, -1853, -1924, -1991
	.dw	 -2054, -2112, -2164, -2210, -2248, -2279, -2299, -2310
	.dw	 -2310, -2298, -2272, -2233, -2179, -2109, -2022, -1917
	.dw	 -1793, -1650, -1486, -1300, -1093,  -862,  -608,  -330
	.dw	   -26,   301,   655,  1036,  1442,  1875,  2335,  2821
	.dw	  3333,  3872,  4436,  5025,  5639,  6277,  6937,  7620
	.dw	  8324,  9048,  9791, 10550, 11326, 12116, 12918, 13731
	.dw	 14552, 15381, 16214, 17051, 17887, 18723, 19554, 20379
	.dw	 21196, 22002, 22795, 23572, 24332, 25072, 25789, 26482
	.dw	 27148, 27786, 28392, 28966, 29506, 30009, 30474, 30900
	.dw	 31284, 31627, 31926, 32181, 32391, 32555, 32672, 32743
	.dw	 32767, 32743, 32672, 32555, 32391, 32181, 31926, 31627
	.dw	 31284, 30900, 30474, 30009, 29506, 28966, 28392, 27786
	.dw	 27148, 26482, 25789, 25072, 24332, 23572, 22795, 22002
	.dw	 21196, 20379, 19554, 18723, 17887, 17051, 16214, 15381
	.dw	 14552, 13731, 12918, 12116, 11326, 10550,  9791,  9048
	.dw	  8324,  7620,  6937,  6277,  5639,  5025,  4436,  3872
	.dw	  3333,  2821,  2335,  1875,  1442,  1036,   655,   301
	.dw	   -26,  -330,  -608,  -862, -1093, -1300, -1486, -1650
	.dw	 -1793, -1917, -2022, -2109, -2179, -2233, -2272, -2298
	.dw	 -2310, -2310, -2299, -2279, -2248, -2210, -2164, -2112
	.dw	 -2054, -1991, -1924, -1853, -1780, -1705, -1628, -1550
	.dw	 -1472, -1394, -1316, -1239, -1164, -1090, -1018,  -948
	.dw	  -880,  -815,  -752,  -692,  -636,  -581,  -530,  -482
	.dw	  -437,  -394,  -355,  -318,  -284,  -252,  -223,  -196
	.dw	  -172,  -150,  -130,  -112,   -96,   -81,   -69,   -57
	.dw	   -48,   -39,   -32,   -26,   -22,   -18,   -15,   -14
	
	.endif


	.elif	FHTN==512
	;====== Lookups for N=512 ======

	;Bit reversion for N=512, byte constants *2
TFHTbrv:	.dw	0*2,256*2,128*2,384*2,64*2,320*2,192*2,448*2
	.dw	32*2,288*2,160*2,416*2,96*2,352*2,224*2,480*2
	.dw	16*2,272*2,144*2,400*2,80*2,336*2,208*2,464*2
	.dw	48*2,304*2,176*2,432*2,112*2,368*2,240*2,496*2
	.dw	8*2,264*2,136*2,392*2,72*2,328*2,200*2,456*2
	.dw	40*2,296*2,168*2,424*2,104*2,360*2,232*2,488*2
	.dw	24*2,280*2,152*2,408*2,88*2,344*2,216*2,472*2
	.dw	56*2,312*2,184*2,440*2,120*2,376*2,248*2,504*2
	.dw	4*2,260*2,132*2,388*2,68*2,324*2,196*2,452*2
	.dw	36*2,292*2,164*2,420*2,100*2,356*2,228*2,484*2
	.dw	20*2,276*2,148*2,404*2,84*2,340*2,212*2,468*2
	.dw	52*2,308*2,180*2,436*2,116*2,372*2,244*2,500*2
	.dw	12*2,268*2,140*2,396*2,76*2,332*2,204*2,460*2
	.dw	44*2,300*2,172*2,428*2,108*2,364*2,236*2,492*2
	.dw	28*2,284*2,156*2,412*2,92*2,348*2,220*2,476*2
	.dw	60*2,316*2,188*2,444*2,124*2,380*2,252*2,508*2
	.dw	2*2,258*2,130*2,386*2,66*2,322*2,194*2,450*2
	.dw	34*2,290*2,162*2,418*2,98*2,354*2,226*2,482*2
	.dw	18*2,274*2,146*2,402*2,82*2,338*2,210*2,466*2
	.dw	50*2,306*2,178*2,434*2,114*2,370*2,242*2,498*2
	.dw	10*2,266*2,138*2,394*2,74*2,330*2,202*2,458*2
	.dw	42*2,298*2,170*2,426*2,106*2,362*2,234*2,490*2
	.dw	26*2,282*2,154*2,410*2,90*2,346*2,218*2,474*2
	.dw	58*2,314*2,186*2,442*2,122*2,378*2,250*2,506*2
	.dw	6*2,262*2,134*2,390*2,70*2,326*2,198*2,454*2
	.dw	38*2,294*2,166*2,422*2,102*2,358*2,230*2,486*2
	.dw	22*2,278*2,150*2,406*2,86*2,342*2,214*2,470*2
	.dw	54*2,310*2,182*2,438*2,118*2,374*2,246*2,502*2
	.dw	14*2,270*2,142*2,398*2,78*2,334*2,206*2,462*2
	.dw	46*2,302*2,174*2,430*2,110*2,366*2,238*2,494*2
	.dw	30*2,286*2,158*2,414*2,94*2,350*2,222*2,478*2
	.dw	62*2,318*2,190*2,446*2,126*2,382*2,254*2,510*2
	.dw	1*2,257*2,129*2,385*2,65*2,321*2,193*2,449*2
	.dw	33*2,289*2,161*2,417*2,97*2,353*2,225*2,481*2
	.dw	17*2,273*2,145*2,401*2,81*2,337*2,209*2,465*2
	.dw	49*2,305*2,177*2,433*2,113*2,369*2,241*2,497*2
	.dw	9*2,265*2,137*2,393*2,73*2,329*2,201*2,457*2
	.dw	41*2,297*2,169*2,425*2,105*2,361*2,233*2,489*2
	.dw	25*2,281*2,153*2,409*2,89*2,345*2,217*2,473*2
	.dw	57*2,313*2,185*2,441*2,121*2,377*2,249*2,505*2
	.dw	5*2,261*2,133*2,389*2,69*2,325*2,197*2,453*2
	.dw	37*2,293*2,165*2,421*2,101*2,357*2,229*2,485*2
	.dw	21*2,277*2,149*2,405*2,85*2,341*2,213*2,469*2
	.dw	53*2,309*2,181*2,437*2,117*2,373*2,245*2,501*2
	.dw	13*2,269*2,141*2,397*2,77*2,333*2,205*2,461*2
	.dw	45*2,301*2,173*2,429*2,109*2,365*2,237*2,493*2
	.dw	29*2,285*2,157*2,413*2,93*2,349*2,221*2,477*2
	.dw	61*2,317*2,189*2,445*2,125*2,381*2,253*2,509*2
	.dw	3*2,259*2,131*2,387*2,67*2,323*2,195*2,451*2
	.dw	35*2,291*2,163*2,419*2,99*2,355*2,227*2,483*2
	.dw	19*2,275*2,147*2,403*2,83*2,339*2,211*2,467*2
	.dw	51*2,307*2,179*2,435*2,115*2,371*2,243*2,499*2
	.dw	11*2,267*2,139*2,395*2,75*2,331*2,203*2,459*2
	.dw	43*2,299*2,171*2,427*2,107*2,363*2,235*2,491*2
	.dw	27*2,283*2,155*2,411*2,91*2,347*2,219*2,475*2
	.dw	59*2,315*2,187*2,443*2,123*2,379*2,251*2,507*2
	.dw	7*2,263*2,135*2,391*2,71*2,327*2,199*2,455*2
	.dw	39*2,295*2,167*2,423*2,103*2,359*2,231*2,487*2
	.dw	23*2,279*2,151*2,407*2,87*2,343*2,215*2,471*2
	.dw	55*2,311*2,183*2,439*2,119*2,375*2,247*2,503*2
	.dw	15*2,271*2,143*2,399*2,79*2,335*2,207*2,463*2
	.dw	47*2,303*2,175*2,431*2,111*2,367*2,239*2,495*2
	.dw	31*2,287*2,159*2,415*2,95*2,351*2,223*2,479*2
	.dw	63*2,319*2,191*2,447*2,127*2,383*2,255*2,511*2
	
	;sin(w), cos(w) values for w=<0,pi/2) for N=512 [fx 1.15]
TFHTsico:	.dw	     0, 32768,   402, 32765,   804, 32758,  1206, 32745
	.dw	  1607, 32728,  2009, 32706,  2410, 32679,  2811, 32647
	.dw	  3211, 32610,  3611, 32568,  4011, 32521,  4409, 32469
	.dw	  4808, 32413,  5205, 32351,  5602, 32285,  5997, 32214
	.dw	  6392, 32138,  6786, 32057,  7179, 31971,  7571, 31881
	.dw	  7961, 31785,  8351, 31685,  8739, 31581,  9126, 31471
	.dw	  9512, 31357,  9896, 31237, 10278, 31114, 10659, 30985
	.dw	 11039, 30852, 11416, 30714, 11793, 30572, 12167, 30425
	.dw	 12539, 30273, 12910, 30117, 13278, 29956, 13645, 29791
	.dw	 14010, 29621, 14372, 29447, 14732, 29269, 15090, 29086
	.dw	 15446, 28898, 15800, 28707, 16151, 28511, 16499, 28310
	.dw	 16846, 28106, 17189, 27897, 17530, 27684, 17869, 27466
	.dw	 18204, 27245, 18537, 27020, 18868, 26790, 19195, 26557
	.dw	 19519, 26319, 19841, 26077, 20159, 25832, 20475, 25583
	.dw	 20787, 25330, 21097, 25073, 21403, 24812, 21706, 24547
	.dw	 22005, 24279, 22301, 24007, 22594, 23732, 22884, 23453
	.dw	 23170, 23170, 23453, 22884, 23732, 22594, 24007, 22301
	.dw	 24279, 22005, 24547, 21706, 24812, 21403, 25073, 21097
	.dw	 25330, 20787, 25583, 20475, 25832, 20159, 26077, 19841
	.dw	 26319, 19519, 26557, 19195, 26790, 18868, 27020, 18537
	.dw	 27245, 18204, 27466, 17869, 27684, 17530, 27897, 17189
	.dw	 28106, 16846, 28310, 16499, 28511, 16151, 28707, 15800
	.dw	 28898, 15446, 29086, 15090, 29269, 14732, 29447, 14372
	.dw	 29621, 14010, 29791, 13645, 29956, 13278, 30117, 12910
	.dw	 30273, 12539, 30425, 12167, 30572, 11793, 30714, 11416
	.dw	 30852, 11039, 30985, 10659, 31114, 10278, 31237,  9896
	.dw	 31357,  9512, 31471,  9126, 31581,  8739, 31685,  8351
	.dw	 31785,  7961, 31881,  7571, 31971,  7179, 32057,  6786
	.dw	 32138,  6392, 32214,  5997, 32285,  5602, 32351,  5205
	.dw	 32413,  4808, 32469,  4409, 32521,  4011, 32568,  3611
	.dw	 32610,  3211, 32647,  2811, 32679,  2410, 32706,  2009
	.dw	 32728,  1607, 32745,  1206, 32758,   804, 32765,   402
	
	.if	FHTW==wHANN
	;Hanning window for N=512 [fx 1.15]
TFHTwdw:	.dw	     0,     1,     4,    11,    19,    30,    44,    60
	.dw	    78,    99,   123,   149,   177,   208,   241,   276
	.dw	   314,   355,   398,   443,   490,   541,   593,   648
	.dw	   705,   765,   826,   891,   957,  1026,  1097,  1171
	.dw	  1247,  1325,  1405,  1488,  1572,  1660,  1749,  1840
	.dw	  1934,  2030,  2128,  2228,  2330,  2435,  2541,  2650
	.dw	  2761,  2873,  2988,  3105,  3224,  3344,  3467,  3592
	.dw	  3718,  3847,  3977,  4110,  4244,  4380,  4517,  4657
	.dw	  4798,  4941,  5086,  5232,  5381,  5530,  5682,  5835
	.dw	  5989,  6146,  6303,  6463,  6623,  6786,  6949,  7114
	.dw	  7281,  7449,  7618,  7788,  7960,  8133,  8308,  8483
	.dw	  8660,  8838,  9017,  9197,  9378,  9560,  9744,  9928
	.dw	 10113, 10300, 10487, 10675, 10864, 11053, 11244, 11435
	.dw	 11627, 11820, 12013, 12207, 12402, 12597, 12793, 12990
	.dw	 13187, 13384, 13582, 13780, 13979, 14178, 14377, 14577
	.dw	 14777, 14977, 15178, 15378, 15579, 15780, 15981, 16182
	.dw	 16383, 16584, 16785, 16986, 17187, 17388, 17588, 17789
	.dw	 17989, 18189, 18389, 18588, 18787, 18986, 19184, 19382
	.dw	 19579, 19776, 19973, 20169, 20364, 20559, 20753, 20946
	.dw	 21139, 21331, 21522, 21713, 21902, 22091, 22279, 22466
	.dw	 22653, 22838, 23022, 23206, 23388, 23569, 23749, 23928
	.dw	 24106, 24283, 24458, 24633, 24806, 24978, 25148, 25317
	.dw	 25485, 25652, 25817, 25980, 26143, 26303, 26463, 26620
	.dw	 26777, 26931, 27084, 27236, 27385, 27534, 27680, 27825
	.dw	 27968, 28109, 28249, 28386, 28522, 28656, 28789, 28919
	.dw	 29048, 29174, 29299, 29422, 29542, 29661, 29778, 29893
	.dw	 30005, 30116, 30225, 30331, 30436, 30538, 30638, 30736
	.dw	 30832, 30926, 31017, 31106, 31194, 31278, 31361, 31441
	.dw	 31519, 31595, 31669, 31740, 31809, 31875, 31940, 32001
	.dw	 32061, 32118, 32173, 32225, 32276, 32323, 32368, 32411
	.dw	 32452, 32490, 32525, 32558, 32589, 32617, 32643, 32667
	.dw	 32688, 32706, 32722, 32736, 32747, 32755, 32762, 32765
	.dw	 32767, 32765, 32762, 32755, 32747, 32736, 32722, 32706
	.dw	 32688, 32667, 32643, 32617, 32589, 32558, 32525, 32490
	.dw	 32452, 32411, 32368, 32323, 32276, 32225, 32173, 32118
	.dw	 32061, 32001, 31940, 31875, 31809, 31740, 31669, 31595
	.dw	 31519, 31441, 31361, 31278, 31194, 31106, 31017, 30926
	.dw	 30832, 30736, 30638, 30538, 30436, 30331, 30225, 30116
	.dw	 30005, 29893, 29778, 29661, 29542, 29422, 29299, 29174
	.dw	 29048, 28919, 28789, 28656, 28522, 28386, 28249, 28109
	.dw	 27968, 27825, 27680, 27534, 27385, 27236, 27084, 26931
	.dw	 26777, 26620, 26463, 26303, 26143, 25980, 25817, 25652
	.dw	 25485, 25317, 25148, 24978, 24806, 24633, 24458, 24283
	.dw	 24106, 23928, 23749, 23569, 23388, 23206, 23022, 22838
	.dw	 22653, 22466, 22279, 22091, 21902, 21713, 21522, 21331
	.dw	 21139, 20946, 20753, 20559, 20364, 20169, 19973, 19776
	.dw	 19579, 19382, 19184, 18986, 18787, 18588, 18389, 18189
	.dw	 17989, 17789, 17588, 17388, 17187, 16986, 16785, 16584
	.dw	 16383, 16182, 15981, 15780, 15579, 15378, 15178, 14977
	.dw	 14777, 14577, 14377, 14178, 13979, 13780, 13582, 13384
	.dw	 13187, 12990, 12793, 12597, 12402, 12207, 12013, 11820
	.dw	 11627, 11435, 11244, 11053, 10864, 10675, 10487, 10300
	.dw	 10113,  9928,  9744,  9560,  9378,  9197,  9017,  8838
	.dw	  8660,  8483,  8308,  8133,  7960,  7788,  7618,  7449
	.dw	  7281,  7114,  6949,  6786,  6623,  6463,  6303,  6146
	.dw	  5989,  5835,  5682,  5530,  5381,  5232,  5086,  4941
	.dw	  4798,  4657,  4517,  4380,  4244,  4110,  3977,  3847
	.dw	  3718,  3592,  3467,  3344,  3224,  3105,  2988,  2873
	.dw	  2761,  2650,  2541,  2435,  2330,  2228,  2128,  2030
	.dw	  1934,  1840,  1749,  1660,  1572,  1488,  1405,  1325
	.dw	  1247,  1171,  1097,  1026,   957,   891,   826,   765
	.dw	   705,   648,   593,   541,   490,   443,   398,   355
	.dw	   314,   276,   241,   208,   177,   149,   123,    99
	.dw	    78,    60,    44,    30,    19,    11,     4,     1

	.elif	FHTW==wHAMM
	;Hamming window for N=512 [fx 1.15]
TFHTwdw:	.dw	  2621,  2622,  2625,  2631,  2639,  2649,  2662,  2676
	.dw	  2693,  2713,  2734,  2758,  2784,  2812,  2843,  2876
	.dw	  2910,  2948,  2987,  3029,  3073,  3119,  3167,  3217
	.dw	  3270,  3325,  3382,  3441,  3502,  3565,  3631,  3698
	.dw	  3768,  3840,  3914,  3990,  4068,  4148,  4230,  4314
	.dw	  4401,  4489,  4579,  4671,  4765,  4861,  4959,  5059
	.dw	  5161,  5265,  5370,  5478,  5587,  5698,  5811,  5926
	.dw	  6042,  6160,  6280,  6402,  6525,  6651,  6777,  6906
	.dw	  7036,  7167,  7300,  7435,  7571,  7709,  7849,  7989
	.dw	  8132,  8275,  8420,  8567,  8715,  8864,  9015,  9167
	.dw	  9320,  9474,  9630,  9787,  9945, 10104, 10264, 10426
	.dw	 10588, 10752, 10917, 11082, 11249, 11417, 11586, 11755
	.dw	 11926, 12097, 12269, 12442, 12616, 12790, 12966, 13142
	.dw	 13318, 13496, 13674, 13852, 14031, 14211, 14391, 14572
	.dw	 14753, 14935, 15117, 15299, 15482, 15665, 15849, 16032
	.dw	 16216, 16400, 16585, 16769, 16954, 17139, 17324, 17509
	.dw	 17694, 17879, 18064, 18248, 18433, 18618, 18803, 18987
	.dw	 19171, 19355, 19539, 19722, 19905, 20088, 20271, 20453
	.dw	 20634, 20815, 20996, 21176, 21356, 21535, 21714, 21892
	.dw	 22069, 22246, 22422, 22597, 22772, 22945, 23118, 23290
	.dw	 23462, 23632, 23802, 23970, 24138, 24305, 24471, 24635
	.dw	 24799, 24962, 25123, 25283, 25443, 25601, 25758, 25913
	.dw	 26068, 26221, 26373, 26523, 26673, 26820, 26967, 27112
	.dw	 27256, 27398, 27539, 27678, 27816, 27952, 28087, 28220
	.dw	 28352, 28482, 28610, 28737, 28862, 28985, 29107, 29227
	.dw	 29345, 29462, 29576, 29689, 29800, 29910, 30017, 30123
	.dw	 30226, 30328, 30428, 30526, 30622, 30716, 30808, 30899
	.dw	 30987, 31073, 31157, 31239, 31319, 31397, 31473, 31547
	.dw	 31619, 31689, 31757, 31822, 31885, 31947, 32006, 32063
	.dw	 32117, 32170, 32221, 32269, 32315, 32359, 32400, 32440
	.dw	 32477, 32512, 32545, 32575, 32603, 32629, 32653, 32675
	.dw	 32694, 32711, 32726, 32738, 32748, 32756, 32762, 32765
	.dw	 32767, 32765, 32762, 32756, 32748, 32738, 32726, 32711
	.dw	 32694, 32675, 32653, 32629, 32603, 32575, 32545, 32512
	.dw	 32477, 32440, 32400, 32359, 32315, 32269, 32221, 32170
	.dw	 32117, 32063, 32006, 31947, 31885, 31822, 31757, 31689
	.dw	 31619, 31547, 31473, 31397, 31319, 31239, 31157, 31073
	.dw	 30987, 30899, 30808, 30716, 30622, 30526, 30428, 30328
	.dw	 30226, 30123, 30017, 29910, 29800, 29689, 29576, 29462
	.dw	 29345, 29227, 29107, 28985, 28862, 28737, 28610, 28482
	.dw	 28352, 28220, 28087, 27952, 27816, 27678, 27539, 27398
	.dw	 27256, 27112, 26967, 26820, 26673, 26523, 26373, 26221
	.dw	 26068, 25913, 25758, 25601, 25443, 25283, 25123, 24962
	.dw	 24799, 24635, 24471, 24305, 24138, 23970, 23802, 23632
	.dw	 23462, 23290, 23118, 22945, 22772, 22597, 22422, 22246
	.dw	 22069, 21892, 21714, 21535, 21356, 21176, 20996, 20815
	.dw	 20634, 20453, 20271, 20088, 19905, 19722, 19539, 19355
	.dw	 19171, 18987, 18803, 18618, 18433, 18248, 18064, 17879
	.dw	 17694, 17509, 17324, 17139, 16954, 16769, 16585, 16400
	.dw	 16216, 16032, 15849, 15665, 15482, 15299, 15117, 14935
	.dw	 14753, 14572, 14391, 14211, 14031, 13852, 13674, 13496
	.dw	 13318, 13142, 12966, 12790, 12616, 12442, 12269, 12097
	.dw	 11926, 11755, 11586, 11417, 11249, 11082, 10917, 10752
	.dw	 10588, 10426, 10264, 10104,  9945,  9787,  9630,  9474
	.dw	  9320,  9167,  9015,  8864,  8715,  8567,  8420,  8275
	.dw	  8132,  7989,  7849,  7709,  7571,  7435,  7300,  7167
	.dw	  7036,  6906,  6777,  6651,  6525,  6402,  6280,  6160
	.dw	  6042,  5926,  5811,  5698,  5587,  5478,  5370,  5265
	.dw	  5161,  5059,  4959,  4861,  4765,  4671,  4579,  4489
	.dw	  4401,  4314,  4230,  4148,  4068,  3990,  3914,  3840
	.dw	  3768,  3698,  3631,  3565,  3502,  3441,  3382,  3325
	.dw	  3270,  3217,  3167,  3119,  3073,  3029,  2987,  2948
	.dw	  2910,  2876,  2843,  2812,  2784,  2758,  2734,  2713
	.dw	  2693,  2676,  2662,  2649,  2639,  2631,  2625,  2622

	.elif	FHTW==wBLAC
	;Blackman window for N=512 [fx 1.15]
TFHTwdw:	.dw	     0,     0,     1,     3,     7,    11,    16,    21
	.dw	    28,    36,    44,    54,    64,    75,    87,   101
	.dw	   115,   130,   146,   163,   181,   200,   220,   241
	.dw	   263,   286,   311,   336,   362,   390,   418,   448
	.dw	   479,   511,   544,   578,   614,   651,   689,   728
	.dw	   769,   811,   854,   899,   945,   992,  1041,  1091
	.dw	  1142,  1195,  1250,  1306,  1363,  1422,  1483,  1545
	.dw	  1608,  1674,  1741,  1809,  1879,  1951,  2025,  2100
	.dw	  2177,  2255,  2336,  2418,  2502,  2588,  2676,  2765
	.dw	  2857,  2950,  3045,  3142,  3241,  3342,  3445,  3550
	.dw	  3656,  3765,  3876,  3988,  4103,  4220,  4339,  4459
	.dw	  4582,  4707,  4834,  4963,  5094,  5227,  5362,  5499
	.dw	  5638,  5780,  5923,  6068,  6216,  6365,  6517,  6671
	.dw	  6826,  6984,  7144,  7305,  7469,  7635,  7802,  7972
	.dw	  8144,  8317,  8493,  8670,  8849,  9030,  9213,  9398
	.dw	  9585,  9773,  9963, 10155, 10349, 10544, 10741, 10940
	.dw	 11140, 11342, 11546, 11750, 11957, 12165, 12374, 12585
	.dw	 12797, 13010, 13224, 13440, 13657, 13875, 14094, 14315
	.dw	 14536, 14758, 14982, 15206, 15431, 15656, 15883, 16110
	.dw	 16338, 16566, 16795, 17025, 17255, 17485, 17716, 17947
	.dw	 18178, 18409, 18641, 18872, 19104, 19335, 19566, 19797
	.dw	 20028, 20259, 20489, 20719, 20949, 21178, 21406, 21634
	.dw	 21861, 22087, 22312, 22537, 22760, 22983, 23204, 23425
	.dw	 23644, 23862, 24078, 24293, 24507, 24719, 24930, 25139
	.dw	 25347, 25552, 25756, 25958, 26158, 26356, 26552, 26746
	.dw	 26938, 27127, 27314, 27499, 27682, 27862, 28040, 28215
	.dw	 28387, 28557, 28724, 28888, 29050, 29209, 29364, 29517
	.dw	 29667, 29814, 29957, 30098, 30235, 30369, 30500, 30627
	.dw	 30752, 30872, 30990, 31103, 31214, 31321, 31424, 31523
	.dw	 31619, 31712, 31800, 31885, 31966, 32043, 32117, 32186
	.dw	 32252, 32314, 32372, 32426, 32476, 32522, 32565, 32603
	.dw	 32637, 32667, 32694, 32716, 32734, 32748, 32758, 32764
	.dw	 32766, 32764, 32758, 32748, 32734, 32716, 32694, 32667
	.dw	 32637, 32603, 32565, 32522, 32476, 32426, 32372, 32314
	.dw	 32252, 32186, 32117, 32043, 31966, 31885, 31800, 31712
	.dw	 31619, 31523, 31424, 31321, 31214, 31103, 30990, 30872
	.dw	 30752, 30627, 30500, 30369, 30235, 30098, 29957, 29814
	.dw	 29667, 29517, 29364, 29209, 29050, 28888, 28724, 28557
	.dw	 28387, 28215, 28040, 27862, 27682, 27499, 27314, 27127
	.dw	 26938, 26746, 26552, 26356, 26158, 25958, 25756, 25552
	.dw	 25347, 25139, 24930, 24719, 24507, 24293, 24078, 23862
	.dw	 23644, 23425, 23204, 22983, 22760, 22537, 22312, 22087
	.dw	 21861, 21634, 21406, 21178, 20949, 20719, 20489, 20259
	.dw	 20028, 19797, 19566, 19335, 19104, 18872, 18641, 18409
	.dw	 18178, 17947, 17716, 17485, 17255, 17025, 16795, 16566
	.dw	 16338, 16110, 15883, 15656, 15431, 15206, 14982, 14758
	.dw	 14536, 14315, 14094, 13875, 13657, 13440, 13224, 13010
	.dw	 12797, 12585, 12374, 12165, 11957, 11750, 11546, 11342
	.dw	 11140, 10940, 10741, 10544, 10349, 10155,  9963,  9773
	.dw	  9585,  9398,  9213,  9030,  8849,  8670,  8493,  8317
	.dw	  8144,  7972,  7802,  7635,  7469,  7305,  7144,  6984
	.dw	  6826,  6671,  6517,  6365,  6216,  6068,  5923,  5780
	.dw	  5638,  5499,  5362,  5227,  5094,  4963,  4834,  4707
	.dw	  4582,  4459,  4339,  4220,  4103,  3988,  3876,  3765
	.dw	  3656,  3550,  3445,  3342,  3241,  3142,  3045,  2950
	.dw	  2857,  2765,  2676,  2588,  2502,  2418,  2336,  2255
	.dw	  2177,  2100,  2025,  1951,  1879,  1809,  1741,  1674
	.dw	  1608,  1545,  1483,  1422,  1363,  1306,  1250,  1195
	.dw	  1142,  1091,  1041,   992,   945,   899,   854,   811
	.dw	   769,   728,   689,   651,   614,   578,   544,   511
	.dw	   479,   448,   418,   390,   362,   336,   311,   286
	.dw	   263,   241,   220,   200,   181,   163,   146,   130
	.dw	   115,   101,    87,    75,    64,    54,    44,    36
	.dw	    28,    21,    16,    11,     7,     3,     1,     0
	
	
	.elif	FHTW==wFLAT
	.equ	FHTWS,1
	;Flattop window for N=512 [fx 1.15]
TFHTwdw:	.dw	   -13,   -13,   -14,   -14,   -15,   -16,   -18,   -20
	.dw	   -22,   -24,   -26,   -29,   -32,   -36,   -39,   -43
	.dw	   -48,   -52,   -57,   -63,   -69,   -75,   -81,   -88
	.dw	   -96,  -104,  -112,  -121,  -130,  -140,  -150,  -161
	.dw	  -172,  -184,  -196,  -209,  -223,  -237,  -252,  -267
	.dw	  -284,  -300,  -318,  -336,  -355,  -374,  -394,  -415
	.dw	  -437,  -459,  -482,  -506,  -530,  -555,  -581,  -608
	.dw	  -636,  -664,  -692,  -722,  -752,  -783,  -815,  -847
	.dw	  -880,  -914,  -948,  -982, -1018, -1053, -1090, -1126
	.dw	 -1164, -1201, -1239, -1277, -1316, -1355, -1394, -1433
	.dw	 -1472, -1511, -1550, -1589, -1628, -1666, -1705, -1743
	.dw	 -1780, -1817, -1853, -1889, -1924, -1958, -1991, -2023
	.dw	 -2054, -2084, -2112, -2139, -2164, -2188, -2210, -2230
	.dw	 -2248, -2265, -2279, -2290, -2299, -2306, -2310, -2312
	.dw	 -2310, -2305, -2298, -2287, -2272, -2255, -2233, -2208
	.dw	 -2179, -2146, -2109, -2067, -2022, -1972, -1917, -1857
	.dw	 -1793, -1724, -1650, -1570, -1486, -1396, -1300, -1199
	.dw	 -1093,  -980,  -862,  -738,  -608,  -472,  -330,  -181
	.dw	   -26,   134,   301,   475,   655,   842,  1036,  1236
	.dw	  1442,  1656,  1875,  2102,  2335,  2575,  2821,  3074
	.dw	  3333,  3599,  3872,  4151,  4436,  4727,  5025,  5329
	.dw	  5639,  5955,  6277,  6604,  6937,  7276,  7620,  7970
	.dw	  8324,  8684,  9048,  9417,  9791, 10168, 10550, 10936
	.dw	 11326, 11719, 12116, 12515, 12918, 13323, 13731, 14141
	.dw	 14552, 14966, 15381, 15797, 16214, 16632, 17051, 17469
	.dw	 17887, 18305, 18723, 19139, 19554, 19967, 20379, 20789
	.dw	 21196, 21600, 22002, 22400, 22795, 23186, 23572, 23954
	.dw	 24332, 24704, 25072, 25433, 25789, 26139, 26482, 26818
	.dw	 27148, 27471, 27786, 28093, 28392, 28684, 28966, 29240
	.dw	 29506, 29762, 30009, 30246, 30474, 30692, 30900, 31097
	.dw	 31284, 31461, 31627, 31782, 31926, 32059, 32181, 32292
	.dw	 32391, 32479, 32555, 32619, 32672, 32713, 32743, 32761
	.dw	 32767, 32761, 32743, 32713, 32672, 32619, 32555, 32479
	.dw	 32391, 32292, 32181, 32059, 31926, 31782, 31627, 31461
	.dw	 31284, 31097, 30900, 30692, 30474, 30246, 30009, 29762
	.dw	 29506, 29240, 28966, 28684, 28392, 28093, 27786, 27471
	.dw	 27148, 26818, 26482, 26139, 25789, 25433, 25072, 24704
	.dw	 24332, 23954, 23572, 23186, 22795, 22400, 22002, 21600
	.dw	 21196, 20789, 20379, 19967, 19554, 19139, 18723, 18305
	.dw	 17887, 17469, 17051, 16632, 16214, 15797, 15381, 14966
	.dw	 14552, 14141, 13731, 13323, 12918, 12515, 12116, 11719
	.dw	 11326, 10936, 10550, 10168,  9791,  9417,  9048,  8684
	.dw	  8324,  7970,  7620,  7276,  6937,  6604,  6277,  5955
	.dw	  5639,  5329,  5025,  4727,  4436,  4151,  3872,  3599
	.dw	  3333,  3074,  2821,  2575,  2335,  2102,  1875,  1656
	.dw	  1442,  1236,  1036,   842,   655,   475,   301,   134
	.dw	   -26,  -181,  -330,  -472,  -608,  -738,  -862,  -980
	.dw	 -1093, -1199, -1300, -1396, -1486, -1570, -1650, -1724
	.dw	 -1793, -1857, -1917, -1972, -2022, -2067, -2109, -2146
	.dw	 -2179, -2208, -2233, -2255, -2272, -2287, -2298, -2305
	.dw	 -2310, -2312, -2310, -2306, -2299, -2290, -2279, -2265
	.dw	 -2248, -2230, -2210, -2188, -2164, -2139, -2112, -2084
	.dw	 -2054, -2023, -1991, -1958, -1924, -1889, -1853, -1817
	.dw	 -1780, -1743, -1705, -1666, -1628, -1589, -1550, -1511
	.dw	 -1472, -1433, -1394, -1355, -1316, -1277, -1239, -1201
	.dw	 -1164, -1126, -1090, -1053, -1018,  -982,  -948,  -914
	.dw	  -880,  -847,  -815,  -783,  -752,  -722,  -692,  -664
	.dw	  -636,  -608,  -581,  -555,  -530,  -506,  -482,  -459
	.dw	  -437,  -415,  -394,  -374,  -355,  -336,  -318,  -300
	.dw	  -284,  -267,  -252,  -237,  -223,  -209,  -196,  -184
	.dw	  -172,  -161,  -150,  -140,  -130,  -121,  -112,  -104
	.dw	   -96,   -88,   -81,   -75,   -69,   -63,   -57,   -52
	.dw	   -48,   -43,   -39,   -36,   -32,   -29,   -26,   -24
	.dw	   -22,   -20,   -18,   -16,   -15,   -14,   -14,   -13	
	.endif

	.else
	.error	"Ivalid value of FHTN!"
	.endif	
