//==========================================================================================================================
// AVR FHT demo
//==========================================================================================================================
// This simple demo communicates with PC application:
//   http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFHT/prog/AVR_FFT_tester_v3.zip
//
// The PC program generates waveforms, sends them into MCU via virtual COM (2Mbd default rate) and then recieves results.
// The result can be either time domain (original waveform with applied window and bit reversion) or frequency domain
// (linear or log scale amplitude spectrum).
//
// Communication protocol (FHT command):
//   1. PC sends 'F' to initiate command
//   2. MCU responds with 'A' (ack)
//   3. PC sends desired mode byte (bit0: 0-time domain, 1-freq domain, bit1: log spectrum for freq domain)
//   4. MCU sends FHTN word (size)
//   5. MCU sends F_CPU dword (in Hz)
//   6. MCU sends window type byte (0-Hann,1-Hamming,2-Blackman,3-Flattop,other-Rectangular)
//   7. PC sends waveform (FHTN signed words)
//   8. MCU does calcualtions and then sends 'D' (done)
//   9. MCU sends measured intervals in F_CPU/8 cycles, 
//     a. word with FHT_out_???() function execution time
//     b. word with FHT_do() function execution time
//     c. word with FHT_input() function execution time
//   10. MCU sends FHTN words of processed data
// 
//  sqrt32f() test command:
//   1. PC sends 'S' to initiate command
//   2. PC sends word with test repetition count N
//   3. PC sends dword value x
//   4. MCU sends word y=FHT_sqrt32c(x) (accurate function)
//   5. MCU sends word y=FHT_sqrt32f(x) (fast function)
//   6. repeat from step 3)
//
//  log10f() test command:
//   1. PC sends 'L' to initiate command
//   2. PC sends word with test repetition count N
//   3. PC sends word value x
//   4. MCU sends signed word y=FHT_log10f(x)
//   5. repeat from step 3)
//
//
// (c) 2012, Stanislav Maslan, s.maslan@seznam.cz
// http://www.elektronika.kvalitne.cz/ATMEL/necoteorie/transformation/AVRFFT/AVRFFT.html
//==========================================================================================================================
#include <avr/io.h>

#include "main.h"
#include "serial.h"
#include "FHT.h"


// rx buffer 
uint8_t rxbuff[FHTN*2];



//==========================================================================================================================
// Timer 1 - interval measurements
//==========================================================================================================================
// reset time value
#define time_res() TCNT1=0

// get time value
#define time_get() TCNT1






//==========================================================================================================================
// Main
//==========================================================================================================================
int main(void)
{
	// directions (there are some unused signals beacause of unused peripherals on my breadboard)
	DDRA=0x00;
	DDRC=0x00;
	DDRB=1<<DWS|1<<DCK|1<<DDT|1<<RWR|1<<LRS|1<<LCK|1<<DCTS;
	DDRD=1<<IORD|1<<IOWR|1<<ROE|1<<RMA|1<<RHA|1<<BCO;

	// init signals
	PORTB=1<<RWR|0<<LCK|0<<DWS|1<<DCTS;
	PORTD=1<<IORD|1<<IOWR|1<<ROE|1<<RMA|1<<RHA|0<<BCO;

	// USART, rx, tx, full speed
	us_setup(0);

	// timer 1 for interval measurements (XTAL/8)
	TCCR1A=0;
	TCCR1B=1<<WGM12|1<<CS11;
		
	// loose trash
	rx_flush();

	// command loop
	while(1)
	{
		// get cmd

		uint8_t cmd=rx_byte();
		if(cmd=='F')
		{
			// ====== do FHT ======
			
			// ack
			tx_byte('A');

			// get mode byte
			uint8_t mode=rx_byte();

			// send FHT size
			tx_word(FHTN);

			// send CPU frequency
			tx_dword(F_CPU);

			// send window code
			tx_byte(FHTW);

			// recieve data to process and fill them into FHT work array
			FHT_rx_data(rxbuff);

			// now do FHT input (windowing, bit reversion)
			time_res();
			//FHT_rx_input();
			FHT_input(rxbuff);
			uint16_t tinp=time_get();

			// now do main transformation if required
			uint16_t tfft=0;
			uint16_t tout=0;
			if(mode&(1<<FREQ_DOM))
			{
				// yeah - do FHT
				time_res();
				FHT_do();
				tfft=time_get();


				// linear or log scale?
				time_res();
				if(mode&(1<<LOG_SCALE))
				{
					// log scale
					FHT_out_log();
				}
				else
				{
					// linear scale
					FHT_out_lin();
				}
				tout=time_get();
			}


			// done
			tx_byte('D');

			// send processing times
			tx_word(tout);
			tx_word(tfft);
			tx_word(tinp);
			
			// send processed data
			FHT_tx_data();
		}
		else if(cmd=='L')
		{
			// ====== do log10f test ======
			
			// ack
			tx_byte('A');

			// do n calcualtions
			uint16_t n=rx_word();
			do{
				// calculate
				tx_word((uint16_t)FHT_log10f(rx_word()));
			}while(--n);
		}
		else if(cmd=='S')
		{
			// ====== do sqrt32f test ======
			
			// ack
			tx_byte('A');

			// do n calcualtions
			uint16_t n=rx_word();
			do{
				// get input
				uint32_t val=rx_dword();
				// send exact sqrt32() result
				tx_word(FHT_sqrt32c(val));
				// send fast sqrt32() result
				tx_word(FHT_sqrt32f(val));

			}while(--n);
		}
		else
		{
			// ====== other command ======
			
			// no ack
			tx_byte('E');
		}

	}	
}
