;compiler: avrasm
	.include "m644def.inc"	;MCU definitions
	.include "macros.asm"	;general macros
	
;------------------------------------------------------------------------------------------------
;	Options
;------------------------------------------------------------------------------------------------
	.equ	XTAL=16000000	;XTAL frequency [Hz]
	.equ	FFTN=256	;FFT size {32,64,128,256,512}
	.equ	FFTwindow=0	;FFT window (0-hanning, 1-hamming, 2-blackman, anything else - rectangular)
	.equ	FFTopt=1	;FFT speed optimalization (0-none (lower size), 1-speed)
	.equ	FFTprec=0	;FFT precision (0-fast, 1-high precision (enable signed correction))
	.equ	FFTfsqrt=0	;FFT sqrt routine (0-slow high precision, 1-fast lower precision)



;------------------------------------------------------------------------------------------------
;	Some macros
;------------------------------------------------------------------------------------------------
	;reset timer 1
	.macro TRes
	ldi	R16,0
	sts	tcnt1h,R16
	sts	tcnt1l,R16
	.endm

	;get timer 1 value
	.macro TGet
	lds	@0,tcnt1l
	lds	@0H,tcnt1h
	.endm
	
	;T=1 if frequency domain selected 
	.macro FfrdT
	lds	R16,FFTtodo
	bst	R16,0
	.endm
	
	;T=1 if log scale enabled
	.macro FlgT
	lds	R16,FFTtodo
	bst	R16,1
	.endm	

;------------------------------------------------------------------------------------------------
;	SRAM variables
;------------------------------------------------------------------------------------------------	
	.dseg
	.org	0x0100
FFTwarr:	.byte	FFTN*2*2	;complex FFT work array [fx 0.16]
FFTtodo:	.byte	1	;to do flags
	


	.cseg
	.org	0x0000
	rjmp	Init
	

;************************************************************************************************
;	Initialization
;************************************************************************************************	
Init:	outi	ddrb,0b00001111	;pin directins
	outi	portb,0b00001111	;pin states

	outiw	sp,ramend	;stack pointer
	
	;USART rx, tx, 8bit, 1 stop, no parity
	stsi	ucsr0a,1<<u2x0
	stsi	ucsr0b,(1<<rxen0)|(1<<txen0)
	stsi	ucsr0c,(1<<ucsz01)|(1<<ucsz00)
	stsiw	ubrr0,0	;2MBd @ 16MHz
	rcall	UsFlush	;loose garbage
	
	;timer 1 - XTAL/8 (for measuring time)
	stsi	tccr1a,0
	stsi	tccr1b,(1<<wgm12)|(1<<cs11)	;XTAL/8
	

	
	;====== wait for command ======
COMwait:	rcall	UsRxByte	;get command code
	cpi	R16,'F'
	brne	COMnF
	;'F' - start FFT command
	ldi	R16,'A'	
	rcall	UsTxByte	;ack
	rcall	COMfft	;do FFT command
	rjmp	COMwait	;wait for next comm
	
COMnF:	cpi	R16,'S'
	brne	COMnS
	;'S' - sqrt test command
	ldi	R16,'A'	
	rcall	UsTxByte	;ack
	rjmp	COMsqrt	;switch to sqrt32f algorithm test mode
	
COMnS:	cpi	R16,'L'
	brne	COMnL
	;'L' - log test command
	ldi	R16,'A'	
	rcall	UsTxByte	;ack
	rjmp	COMlog	;switch to log10f algorithm test mode
	
COMnL:	;invalid command code
	ldi	R16,'E'	
	rcall	UsTxByte	;err
	rjmp	COMwait	;and wait for next comm
	
	
	
;------------------------------------------------------------------------------------------------
;	FFT command, communication protocol:
;	1.  recieve "to do" byte, where:
;	      bit0 - time/frequency domain, bit1 - linear/log scale
;	2.  send FFTN word (selected FFT size)
;	3.  send XTAL dword (XTAL frequency)
;	4.  send FFTwindow byte (selected window)
;	5.  recieve FFTN words - signal to process:
;	      stored to re{FFTwarr[]}
;	6.  send 'D' byte (synchronization char.)
;	7.  send "FFTinit" exec. time word (actualy XTAL/8 cycles, not time)
;	8.  send "FFTditr2" exec. time word (actualy XTAL/8 cycles, not time)
;	9.  send "FFTout" exec. time word (actualy XTAL/8 cycles, not time)
;	10. send FFTN words - processed data:
;	      time domain: all re{FFTwarr[]} values
;	      freq. domain: first half of spectrum FFTwarr[] (both re{} and im{} parts)
;------------------------------------------------------------------------------------------------
COMfft:	rcall	UsRxByte
	sts	FFTtodo,R16	;store "to do" flags
	rcall	FFTrxData	;recieve data to process

	;=== here are called FFT algorith parts ===
	cbi	portb,3

	;--- FFT initialization ---
	cbi	portb,0
	TRes
	rcall	FFTinit	;--- bit reversion and window application
	TGet	R16
	pushw	R16
	sbi	portb,0

	;--- main FFT transformation ---
	cbi	portb,1
	TRes
	FfrdT
	brtc	PC+1+1	;skip if time domain mode selected
	rcall	FFTditr2	;--- calculate main FFT dit radix 2
	TGet	R16
	pushw	R16
	sbi	portb,1

	;--- FFT output (amplitude spectum, eventualy in log scale) ---
	cbi	portb,2
	TRes
	FfrdT
	brtc	COMtimDom	;skip if time domain mode selected
	;select liner or log scale
	FlgT
	brts	PC+1+1+1
	rcall	FFToutLin	;--- create linear amplitude spectrum
	rjmp	PC+1+1
	rcall	FFToutLog	;--- create log scale amplitude spectrum	
COMtimDom:	TGet	R16
	pushw	R16
	sbi	portb,2

	sbi	portb,3
		
	;send processing done character (just for synchronisation)
	ldi	R16,'D'
	rcall	UsTxByte
	
	;send measured times
	popw	R16
	rcall	UsTxWord
	popw	R16
	rcall	UsTxWord
	popw	R16
	rcall	UsTxWord

	;and finaly send processed data back
	FfrdT
	rcall	FFTtxData
	ret
	

;------------------------------------------------------------------------------------------------
;	"sqrt32f" test command - endless loop, protocol:
;	1. recieve dword to sqrt
;	2. send word result
;------------------------------------------------------------------------------------------------	
COMsqrt:	rcall	UsRxWord
	movw	R2,R16
	rcall	UsRxWord
	movw	R4,R16
	pushd	R2
	rcall	sqrt32c
	rcall	UsTxWord
	popd	R2
	rcall	sqrt32f
	rcall	UsTxWord
	rjmp	COMsqrt	;for ever
	

;------------------------------------------------------------------------------------------------
;	"log10f" test command - endless loop, protocol:
;	1. recieve word to log()
;	2. send word result
;------------------------------------------------------------------------------------------------
COMlog:	rcall	UsRxWord
	rcall	log10f
	rcall	UsTxWord
	rjmp	COMlog	;for ever	



	
;************************************************************************************************
;	USART routines
;************************************************************************************************
;================================================================================================
;	Flush recieve buffer data
;================================================================================================
UsFlush:	lds	R16,udr0
	lds	R16,udr0
	ret	
	
;================================================================================================
;	Send byte R16
;================================================================================================
UsTxByte:	push	R16
UsTxByteL:	rjsc	ucsr0a,udre0,UsTxByteL
	pop	R16
	sts	udr0,R16
	ret
	
;================================================================================================
;	Send word R16:R17
;================================================================================================
UsTxWord:	rcall	UsTxByte
	mov	R16,R17
	rjmp	UsTxByte
	
;================================================================================================
;	Send dword R16:R17:R18:R19
;================================================================================================
UsTxDWord:	rcall	UsTxByte
	mov	R16,R17
	rcall	UsTxByte
	mov	R16,R18
	rcall	UsTxByte
	mov	R16,R19
	rjmp	UsTxByte	

;================================================================================================
;	Get byte into R16
;================================================================================================	
UsRxByte:	rjsc	ucsr0a,rxc0,UsRxByte
	lds	R16,udr0
	ret

;================================================================================================
;	Get word into R16:R17
;================================================================================================	
UsRxWord:	rcall	UsRxByte
	push	R16
	rcall	UsRxByte
	mov	R17,R16
	pop	R16	
	ret
	




;************************************************************************************************
;	Other module includes
;************************************************************************************************
	.include "math.asm"	;some math routines
	.include "FFTditR2.asm"	;FFT routines
	